import xml.etree.ElementTree as ET
from sets import Set

class XML:
    def __init__(self, input_file="salad-recipes.xml", print_process=False, default_value="_"):
        self.xml_file = input_file
        self.print_process = print_process
        self.default_value = default_value

    # Goes through XML file. Recognizes id, title, ingredients, preparation steps and exclude for diet tags
    def get_recipes(self):
        root = ET.parse(self.xml_file).getroot()
        recipes = []
        for recipe in root.findall('recipe'):
            recipes.append(self.get_recipe(recipe))
        return recipes

    def get_unique_ingredients(self):
        ingredients = []
        for recipe in self.get_recipes():
            for ingredient in recipe["ingredients"]:
                ingredients.append(ingredient[0].lower())

        unique_ingredients = Set(ingredients)
        return unique_ingredients

    # Get all tag values on recipe
    # To recognize the attributes "food", "unit" and "quantity" inside of <ingredient>, print(ingredient.attrib)
    # To recognize the text itself for the ingredient: print(ingredient.text)
    def get_recipe(self, recipe):
        try:
            id = recipe.find("recipeid").text
        except:
            self.pp("No recipe id found in xml")
            id = self.default_value
        title = recipe.find("title").text
        ingredients = []
        for ingredient in recipe.find("ingredients"):
            try:
                ingredients.append((ingredient.attrib["ingredient"].replace("_", " "), ingredient.text.replace('"', "'")))
            except:
                ingredients.append((ingredient.attrib["food"].replace("_", " "), ingredient.text.replace('"', "'")))
                #i = u''.join((ingredient.text)).encode('utf-8').strip()
                #ingredients.append((i, i))
        preparation_steps = []
        for step in recipe.find("preparation"):
            preparation_steps.append(step.text)
        exclude_for = []
        try:
            for exclude_for_diet in recipe.find("diet"):
                exclude_for.append(exclude_for_diet.text)
        except:
            self.pp("No exclude diet found in xml")
        try:
            self.pp("Id: " + id)
        except:
            self.pp("No ID")
        self.pp("Title: " + title)
        try:
            self.pp("Ingredients:")
            for ingredient in ingredients:
                self.pp("- " + str(ingredient))
        except:
            self.pp("Can not print ingredient")
        self.pp("Preparation:")
        for step in preparation_steps:
            self.pp("- " + step)
        try:
            self.pp("Exclude for Diet:")
            for diet in exclude_for:
                self.pp("- " + diet)
        except:
            self.pp("No Exclude Diet")

        return {'recipeId': id, 'title': title, 'ingredients': ingredients, 'preparation': preparation_steps, 'excludeFor': exclude_for}

    # Print function that only prints if self.print_process is set to True
    def pp(self, text):
        if self.print_process:
            print(text)





