import time
from Model import Attribute_model, myCBRModel
from XML import XML
from sets import Set
import operator


class XML_to_CSV_parser:
    def __init__(self):
        self.output_file = "output/model-sandwich.csv"
        self.default_attribute_value = ""
        self.header_attributes = ['Basic','Carbs','CaseId', 'Drinks', 'Fish', 'Fruit', 'Meat', 'Milk', 'Minor Ingredient', 'Oil and Fat', 'Spice and Herb', 'Supplement', 'Title', 'Vegetable']
        self.print_process = True
        xml_file = XML(input_file='resources/sandwich-recipes.xml', print_process=self.print_process, default_value=self.default_attribute_value)

        self.xml_recipes = xml_file.get_recipes()
        self.start_time = time.time()
        self.ingredients_without_model_match = []
        self.mycbr_model = list(Attribute_model)
        self.all_ingredients = []
        self.main()

        self.print_all_unique_ingredients()
        self.print_ingredients_without_match()

    # Fills rows on correct csv format with data both from the fetched xml file and from the mycbr model
    # creates a csv file filled with title and ingredient list
    def main(self):
        output_rows = [self.header_attributes]
        for recipe in self.xml_recipes:
            row = [self.default_attribute_value]*len(self.header_attributes)
            try:
                #row[2] = recipe["recipeId"]
                #print(Attribute_model.get_column_number(Attribute_model.CASEID()))
                #print(Attribute_model.get_column_number(Attribute_model.TITLE()))
                row[Attribute_model.get_caseid_column_number()] = recipe["recipeId"]
            except:
                self.pp("No recipe id found in xml")
            # row[3] = recipe["title"]
            row[Attribute_model.get_title_column_number()] = recipe["title"]
            ingredients_list = []
            ingredients = ""
            for ing in recipe["ingredients"]:
                self.all_ingredients.append(ing[0])
                ingredients +=ing[0] + ","
                ingredients_list.append(ing[0])
            #row[7] = ingredients
            updated_row = self.update_row_with_model(row, ingredients_list)
            output_rows.append(updated_row)
        self.write_csv(output_rows)

    def print_all_unique_ingredients(self):
        ingredient_with_category = dict()
        print(str(len(self.all_ingredients)) + " unique ingredients in XML file")
        for ing in Set(self.all_ingredients):
            category = myCBRModel.match_ingredient(ing)
            if category in ingredient_with_category:
                ingredient_with_category[category].append(ing)
            else:
                ingredient_with_category[category] = [ing]
        for key, value in ingredient_with_category.items():
            print("")
            print(key)
            for ingredient in value:
                print(ingredient)

    def print_ingredients_without_match(self):
        self.pp("")
        self.pp("Ingredients without match (" + str(len(self.ingredients_without_model_match)) + "):")
        for ingredient in self.ingredients_without_model_match:
            self.pp(ingredient)
        self.pp("")
        self.pp("... Parsing completed")
        self.pp("... CSV file saved as " + self.output_file)
        self.pp("--- %s seconds ---" % (time.time() - self.start_time))

    def write_csv(self, output_rows):
        csv_file = open(self.output_file, 'w')
        for row in output_rows:
            for i in range(len(self.header_attributes)):
                column = row[i]
                if type(column) is list:
                    comma_separated_column = ""
                    for ingredient in column:
                        comma_separated_column+=ingredient + ","
                    column = comma_separated_column

                if i == len(self.header_attributes)-1:
                    csv_file.write('%s' % column)
                else:
                    csv_file.write('%s;' % column)
            csv_file.write('\n')
        csv_file.close()

    def update_row_with_model(self, row, ingredients):
        #current_row = copy.deepcopy(row)
        self.pp("")
        self.pp("MATCH INGREDIENTS:")
        for ingredient in ingredients:
            ingredient = ingredient.lower()
            column_match = myCBRModel.match_ingredient(ingredient)
            if column_match == -1:
                try:
                    self.ingredients_without_model_match.append(ingredient.encode("utf-8").lower())
                except:
                    self.ingredients_without_model_match.append(ingredient.lower())
                self.pp(str(ingredient) + ": " + str(0))
            else:
                self.pp(str(ingredient) + ": " + str(1))
                if row[column_match] == self.default_attribute_value:
                    row[column_match] = [ingredient]
                else:
                    try:
                        row[column_match].append(ingredient)
                    except:
                        print("Failed adding ingredient" + str(ingredient) + " to column number " + str(column_match) + ". Most likely because it is not a list")
                        raise
        return row

    # Print function that only prints if self.print_process is set to True
    def pp(self, text):
        if self.print_process:
            print(text)

XML_to_CSV_parser()






