import xml.etree.ElementTree as ET
import time
import operator


class TsGenerator:
    def __init__(self):
        self.print_process = True
        self.input_file = "resources/sandwich-recipes.xml"
        self.output_file = "output/frontend_sandwich.ts"
        self.start_time = time.time()
        self.main()

    # Goes through XML files. Recognizes id, title, ingredients, preparation steps and exclude for diet tags
    # Generates an ts file with desired front-end format
    def main(self):
        root = ET.parse(self.input_file).getroot()
        recipes = []
        for recipe_member in root.findall('recipe'):
            self.pp("")
            self.pp("-------------------")
            recipes.append(self.get_recipe_from_xml(recipe_member))

        with open(self.output_file, "w") as ts_file:
            ts_file.write("import{Recipe, KeyValuePair}from '../home.model';\n" )
            ts_file.write("\n")
            ts_file.write("export const RECIPES: Recipe[] = [\n")

            for recipe in recipes:
                ts_file.write("\t{\n")
                try:
                    ts_file.write('\t\tid: "' + recipe['recipeId'] + '",\n')
                except:
                    self.pp("No recipe id in xml")
                ts_file.write('\t\ttitle: "' + recipe['title'] + '",\n')
                ts_file.write("\t\tingredients: [")
                for ingredient in recipe['ingredients']:
                    ts_file.write('new KeyValuePair("' + str(ingredient[0]) + '", "' + str(ingredient[1]) + '")')
                    if ingredient is not recipe['ingredients'][-1]:
                        ts_file.write(", ")
                    else:
                        ts_file.write("],\n")

                ts_file.write("\t\tpreparationSteps: [")
                for step in recipe['preparation']:
                    ts_file.write('"' + str(step) + '"')
                    if step is not recipe['preparation'][-1]:
                        ts_file.write(", ")
                    else:
                        ts_file.write("],\n")

                #ts_file.write("\t\tpreparationSteps: " + str(recipe['preparation']).replace("'", '"') + ",\n")
                ts_file.write("\t\texcludeFor: " + str(recipe['excludeFor']).replace("'", '"') + "\n")

                if recipe is not recipes[-1]:
                    ts_file.write("\t},\n")
                else:
                    ts_file.write("\t}\n")

            ts_file.write("];\n")

        self.pp("-------------------")
        self.pp("")
        self.pp("... Parsing completed")
        self.pp("... TypoScript file saved as " + self.output_file + " (" + str(len(recipes)) + " recipes)")
        self.pp("--- %s seconds ---" % (time.time() - self.start_time))


    # Get all tag values on recipe
    def get_recipe_from_xml(self, recipe_member):
        try:
            id = recipe_member.find("recipeid").text
        except:
            self.pp("No recipe id found in xml")
            id = None
        title = recipe_member.find("title").text
        ingredients = []
        for ingredient in recipe_member.find("ingredients"):
            try:
                ingredients.append((ingredient.attrib["ingredient"].replace("_", " ").lower(), ingredient.text.replace('"', "'")))
            except:
                ingredients.append((ingredient.attrib["food"].replace("_", " ").lower(), ingredient.text.replace('"', "'")))
        preparation_steps = []
        for step in recipe_member.find("preparation"):
            preparation_steps.append(step.text)
        exclude_for = []
        try:
            for exclude_for_diet in recipe_member.find("diet"):
                exclude_for.append(exclude_for_diet.text)
        except:
            "No exclude diet found in xml"

        self.pp("Id: " + str(id))
        self.pp("Title: " + title)
        self.pp("Ingredients:")
        for ingredient in ingredients:
            self.pp("- " + str(ingredient))
        self.pp("Preparation:")
        for step in preparation_steps:
            self.pp("- " + step)
        self.pp("Exclude for Diet:")
        for diet in exclude_for:
            self.pp("- " + diet)

        return {'recipeId': id, 'title': title, 'ingredients': ingredients, 'preparation': preparation_steps, 'excludeFor': exclude_for}

    # Print function that only prints if self.print_process is set to True
    def pp(self, text):
        if self.print_process:
            print(text)


TsGenerator()






