import xml.etree.ElementTree as ET
import csv
import requests
import time
from Model import Attribute_model, myCBRModel

# 1) Run the Jar file to be able to access the myCbr models
# 2) Set the input (txt file with one ingredient per line. Default: "ingredients_c.txt)
# 3) Run program in terminal. You'll get a two lists. One lists with ingredients that are already in the model and one with ingredients that
# needs to be added manually

# TEST: To test that the ingredients you add manually actually are added, you need to run -mvn clean install- and start the server over again, then
# you will see the changes

class processIngredients:
    def __init__(self):
        self.text_file = open("ingredients/all_ingredients_fixed.txt", "r")
        self.print_process = True
        self.start_time = time.time()
        self.mycbr_model = list(Attribute_model)
        ingredients = self.strip_ingredients()
        self.ingredients_with_match, self.ingredients_without_match = myCBRModel.match_ingredients_with_mycbr_model(ingredients)
        self.ingredient_frequency = self.find_duplicates()
        self.print_results()
        for name, member in Attribute_model.__members__.items():
            #if name == "BASIC":
            if member.value:
                for i in member.value:
                    if "light" in i or "reduced" in i:
                        print i + ": " + str(name)
        # leaves
        # seeds
        # reduced-cal
        # light

    def print_results(self):
        print("MATCHED (" + str(len(self.ingredients_with_match)) + "): ")
        for ingredient in self.ingredients_with_match:
            print ingredient

        print("")
        print("NO MATCH (" + str(len(self.ingredients_without_match)) + "). ADD MANUALLY.")
        for ingredient in self.ingredients_without_match:
            print ingredient

        print("")
        print("DUPLICATES: ")
        for ingredient in self.ingredient_frequency:
            # print (word for word in self.ingredient_frequency[ingredient] if len(self.ingredient_frequency[ingredient]) > 1)
            if len(self.ingredient_frequency[ingredient]) > 1:
                print ""
                print(str(ingredient) + ": - "),
                for accompanying_attribute in self.ingredient_frequency[ingredient]:
                    print str(accompanying_attribute) + " -",

    # Finds duplicates in myCbr model
    # Creates dictionary with ingredient name as key and the accompanying attributes as values
    # E.g: Sugar: BASIC, MINOR_INGREDIENT
    def find_duplicates(self):
        ingredient_frequency = dict()
        for name, member in Attribute_model.__members__.items():
            if member.value is not None:
                for i in member.value:
                    ingredient = i.lower()
                    if ingredient in ingredient_frequency:
                        ingredient_frequency[ingredient].append(name)
                    else:
                        ingredient_frequency[ingredient] = [name]
        return ingredient_frequency

    # RulesInit
    def translate_with_rules(self, i):
        i = i.lower()
        if i == "chicken breasts":
            return "chicken breast fillet"
        elif i == "tomatoes":
            return "tomato"
        elif i == "cherry tomatoes":
            return "cherry tomato"
        elif i == "vine ripened tomatoes":
            return "vine ripened tomato"
        elif i == "bing cherries":
            return "bing cherry"
        elif i == "red-skinned potatoes":
            return "red potato"
        elif i == "lite mayonnaise" or i == "reduced fat mayonnaise" or i == "reduced-fat mayonnaise":
            return "light mayonnaise"
        elif i == "lowfat sour cream" or i == "lite sour cream" or i == "reduced fat sour cream":
            return "low fat sour cream"
        elif i == "hard boiled eggs" or i == "hard-boiled eggs" or i == "hard-cooked eggs" or i == "hard cooked eggs" or i == "hard cooked egg" or i == "hard-cooked egg":
            return "hard boiled egg"

        plurals = ["raisins", "green beans", "scallions", "cucumbers", "eggs", "peanuts", "roasted peanuts", "walnuts", "sesame seeds",
                   "ripe olives", "corn kernelse", "green onions", "red apples", "bean sprouts", "onions", "oranges", "cilantro leaves",
                   "raw beets", "capers", "nicoise olives", "onions", "hard-cooked eggs", "marshmallows", "sweet pickles", "lima beans",
                   "almonds", "kidney beans", "white beans", "yellow beans", "plums", "nectarines", "pecans"]

        # Skip "s" at the end of plurals
        for plural_world in plurals:
            if i == plural_world:
                return i[:-1]
        else:
            return i


    # Takes in all ingredients from textfile and returns a list with (almost) "clean" ingredients. That is, without quantities and units etc.
    def strip_ingredients(self):
        lines = self.text_file.readlines()

        for i in range(len(lines)):
            if str(lines[i][0:4]) == "1 ts" or str(lines[i][0:4]) == "2 ts" or str(lines[i][0:4]) == "3 ts" or str(lines[i][0:4]) == "4 ts" \
                    or str(lines[i][0:4]) == "1 tb" or str(lines[i][0:4]) == "2 tb" or str(lines[i][0:4]) == "3 tb" or str(lines[i][0:4]) == "4 tb" \
                    or str(lines[i][0:4]) == "1/4 " or str(lines[i][0:4]) == "1/2 " or str(lines[i][0:4]) == "1/3 " or str(lines[i][0:4]) == "3/4 " or str(lines[i][0:4]) == "2/3 " \
                    or str(lines[i][0:4]) == "1 c " or str(lines[i][0:4]) == "2 c " or str(lines[i][0:4]) == "3 c " or str(lines[i][0:4]) == "4 c " or str(lines[i][0:4]) == "5 c " or str(lines[i][0:4]) == "6 c " \
                    or str(lines[i][0:4]) == "1 ds":
                lines[i] = lines[i][4:]
            elif str(lines[i][3:5]) == "oz" or str(lines[i][3:5]) == "lg":
                lines[i] = lines[i][5:]
            elif str(lines[i][2:4]) == "oz" or str(lines[i][2:4]) == "lg":
                lines[i] = lines[i][4:]
            elif str(lines[i][0:1]) == "0" or str(lines[i][0:1]) == "1" or str(lines[i][0:1]) == "2" or str(lines[i][0:1]) == "3" or str(lines[i][0:1]) == "4" or str(lines[i][0:1]) == "5" or str(lines[i][0:1]) == "6" or str(lines[i][0:1]) == "7" or str(lines[i][0:1]) == "8":
                lines[i] = lines[i][1:]

        for i in range(len(lines)):
            if str(lines[i][0:1]) == " " or str(lines[i][0:1]) == "0" or str(lines[i][0:1]) == "1" or str(lines[i][0:1]) == "2":
                lines[i] = lines[i][1:]
            elif str(lines[i][0:3]) == "ts ":
                lines[i] = lines[i][3:]
            elif str(lines[i][0:2]) == "c ":
                lines[i] = lines[i][2:]

        for i in range(len(lines)):
            if str(lines[i][0:1]) == " ":
                lines[i] = lines[i][1:]
            elif str(lines[i][0:2]) == "c ":
                lines[i] = lines[i][2:]
            elif str(lines[i][0:3]) == "pk " or str(lines[i][0:3]) == "sl ":
                lines[i] = lines[i][3:]
            elif str(lines[i][0:4]) == "cn (" or str(lines[i][0:4]) == "Jar ":
                lines[i] = lines[i][9:]
            elif str(lines[i][0:5]) == "1/2 c":
                lines[i] = lines[i][5:]
            elif str(lines[i][0:6]) == "3/4 oz":
                lines[i] = lines[i][6:]

        for i in range(len(lines)):
            if str(lines[i][0:1]) == " ":
                lines[i] = lines[i][1:]
            elif str(lines[i][0:2]) == ") ":
                lines[i] = lines[i][2:]
            elif str(lines[i][0:3]) == "sl ":
                lines[i] = lines[i][3:]

        # Strip the \n line shift
        # Translate with rules, e.g., remove plurals
        for i in range(len(lines)):
            if not lines[i] == lines[-1]:
                lines[i] = lines[i][:-1]
            lines[i] = self.translate_with_rules(lines[i])

        self.text_file.close()
        return lines


processIngredients()





