package no.ntnu.mycbr.rest;

import io.swagger.annotations.*;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.CrossOrigin;
import java.util.HashMap;

@CrossOrigin
@RestController
public class CBRController {

    // Similar cases - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    @ApiOperation(value = "getAdaptedSimilarCases", nickname = "getAdaptedSimilarCases")
    @RequestMapping(method = RequestMethod.POST, path = "/adaptation", produces = "application/json")
    @ApiResponses(value = {
            @ApiResponse(code = 200, message = "Success", response = Query.class),
            @ApiResponse(code = 401, message = "Unauthorized"),
            @ApiResponse(code = 403, message = "Forbidden"),
            @ApiResponse(code = 404, message = "Not Found"),
            @ApiResponse(code = 500, message = "Failure")})
    public Query getAdaptedSimilarCases(@RequestParam(value = "casebase", defaultValue = "Sandwich") String casebase,
                                        @RequestParam(value = "concept name", defaultValue = "Cookery") String concept,
                                        @RequestParam(required = false, value = "no of returned cases", defaultValue = "-1") int k,
                                        @RequestBody(required = true) HashMap<String, HashMap<String, Object>> queryContent) {
        return new Query(casebase, concept, queryContent, k);
    }

    @ApiOperation(value = "getSimilarCases", nickname = "getSimilarCases")
    @RequestMapping(method = RequestMethod.POST, path = "/retrieval", produces = "application/json")
    @ApiResponses(value = {
            @ApiResponse(code = 200, message = "Success", response = Query.class),
            @ApiResponse(code = 401, message = "Unauthorized"),
            @ApiResponse(code = 403, message = "Forbidden"),
            @ApiResponse(code = 404, message = "Not Found"),
            @ApiResponse(code = 500, message = "Failure")})
    public Query getSimilarCases(@RequestParam(value = "casebase", defaultValue = "Sandwich") String casebase,
                                 @RequestParam(value = "concept name", defaultValue = "Cookery") String concept,
                                 @RequestParam(required = false, value = "no of returned cases", defaultValue = "-1") int k,
                                 @RequestBody(required = true) HashMap<String, HashMap<String, Object>> queryContent) {
        return new Query(casebase, concept, queryContent, k, false);
    }

    @ApiOperation(value = "getSimilarCasesByID", nickname = "getSimilarCasesByID")
    @RequestMapping(method = RequestMethod.GET, path = "/retrievalByID", produces = "application/json")
    @ApiResponses(value = {
            @ApiResponse(code = 200, message = "Success", response = Query.class),
            @ApiResponse(code = 401, message = "Unauthorized"),
            @ApiResponse(code = 403, message = "Forbidden"),
            @ApiResponse(code = 404, message = "Not Found"),
            @ApiResponse(code = 500, message = "Failure")})
    public Query getSimilarCases(@RequestParam(value = "casebase", defaultValue = "Salad") String casebase,
                                 @RequestParam(value = "concept", defaultValue = "Cookery") String concept,
                                 @RequestParam(value = "caseID", defaultValue = "Cookery0") String caseID) {
        return new Query(casebase, concept, caseID);
    }

    // Case - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    @ApiOperation(value = "getCase", nickname = "getCase")
    @RequestMapping(method = RequestMethod.GET, value = "/case", headers = "Accept=application/json")
    @ApiResponses(value = {
            @ApiResponse(code = 200, message = "Success", response = Case.class),
            @ApiResponse(code = 401, message = "Unauthorized"),
            @ApiResponse(code = 403, message = "Forbidden"),
            @ApiResponse(code = 404, message = "Not Found"),
            @ApiResponse(code = 500, message = "Failure")})
    public Case getCase(@RequestParam(value = "caseID", defaultValue = "Cookery0") String caseID) {
        return new Case(caseID);
    }

    @ApiOperation(value = "addCaseAndRecipe", nickname = "addCaseAndRecipe")
    @RequestMapping(method = RequestMethod.POST, path = "/addcase", produces = "application/json")
    @ApiResponses(value = {
            @ApiResponse(code = 200, message = "Success", response = String.class),
            @ApiResponse(code = 401, message = "Unauthorized"),
            @ApiResponse(code = 403, message = "Forbidden"),
            @ApiResponse(code = 404, message = "Not Found"),
            @ApiResponse(code = 500, message = "Failure")})
    public String addCaseAndRecipe(@RequestParam(value = "casebase", defaultValue = "Sandwich") String casebase,
                                   @RequestParam(value = "concept name", defaultValue = "Cookery") String concept,
                                   @RequestParam(value = "recipe", defaultValue = "") String recipe,
                                   @RequestBody(required = true) HashMap<String, Object> newCase) {
        return new AddCaseAndRecipe(casebase, concept, recipe, newCase).addNewCase();
    }

    // Recipe - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    @ApiOperation(value = "getRecipe", nickname = "getRecipe")
    @RequestMapping(method = RequestMethod.GET, path = "/recipe", produces = "application/json")
    @ApiResponses(value = {
            @ApiResponse(code = 200, message = "Success", response = Recipe.class),
            @ApiResponse(code = 401, message = "Unauthorized"),
            @ApiResponse(code = 403, message = "Forbidden"),
            @ApiResponse(code = 404, message = "Not Found"),
            @ApiResponse(code = 500, message = "Failure")})
    public Recipe getRecipe(
            @RequestParam(value = "recipeId", defaultValue = "1") String recipeId) {
        return new Recipe(recipeId);
    }

    // myCBR model attributes and values - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    @ApiOperation(value = "getAttributes", nickname = "getAttributes")
    @RequestMapping(method = RequestMethod.GET, value = "/attributes", headers = "Accept=application/json")
    @ApiResponses(value = {
            @ApiResponse(code = 200, message = "Success", response = ApiResponse.class),
            @ApiResponse(code = 401, message = "Unauthorized"),
            @ApiResponse(code = 403, message = "Forbidden"),
            @ApiResponse(code = 404, message = "Not Found"),
            @ApiResponse(code = 500, message = "Failure")})
    public Attribute getAttributes(@RequestParam(value = "concept name", defaultValue = "Cookery") String concept) {
        return new Attribute(concept);
    }

    @ApiOperation(value = "getValueRange", nickname = "getValueRange")
    @RequestMapping(method = RequestMethod.GET, value = "/values", headers = "Accept=application/json")
    @ApiResponses(value = {
            @ApiResponse(code = 200, message = "Success", response = ValueRange.class),
            @ApiResponse(code = 401, message = "Unauthorized"),
            @ApiResponse(code = 403, message = "Forbidden"),
            @ApiResponse(code = 404, message = "Not Found"),
            @ApiResponse(code = 500, message = "Failure")})
    public ValueRange getValueRange(@RequestParam(value = "concept name", defaultValue = "Cookery") String concept,
                                    @RequestParam(value = "attribute name", defaultValue = "Basic") String attributeName) {
        return new ValueRange(concept, attributeName);
    }

    // Quiz related operations - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    @ApiOperation(value = "quizAnswer", nickname = "quizAnswer")
    @RequestMapping(method = RequestMethod.GET, path = "/submitquizanswer", produces = "application/json")
    @ApiResponses(value = {
            @ApiResponse(code = 200, message = "Success", response = String.class),
            @ApiResponse(code = 401, message = "Unauthorized"),
            @ApiResponse(code = 403, message = "Forbidden"),
            @ApiResponse(code = 404, message = "Not Found"),
            @ApiResponse(code = 500, message = "Failure")})
    public String addQuizAnswer(@RequestParam(value = "casename", defaultValue = "") String casename,
                                @RequestParam(value = "createdByAi", defaultValue = "") Boolean createdByAi,
                                @RequestParam(value = "answer", defaultValue = "") String answer) {
        return new QuizAnswer(casename, createdByAi, answer).addQuizAnswer();
    }

    // Currently unused API operations - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    @ApiOperation(value = "getConcept", nickname = "getConcept")
    @RequestMapping(method = RequestMethod.GET, path = "/concepts", produces = "application/json")
    @ApiResponses(value = {
            @ApiResponse(code = 200, message = "Success", response = ConceptName.class),
            @ApiResponse(code = 401, message = "Unauthorized"),
            @ApiResponse(code = 403, message = "Forbidden"),
            @ApiResponse(code = 404, message = "Not Found"),
            @ApiResponse(code = 500, message = "Failure")})
    public ConceptName getConcept() {
        return new ConceptName();
    }

    @ApiOperation(value = "getCaseBases", nickname = "getCaseBases")
    @RequestMapping(method = RequestMethod.GET, path = "/casebase", produces = "application/json")
    @ApiResponses(value = {
            @ApiResponse(code = 200, message = "Success", response = CaseBases.class),
            @ApiResponse(code = 401, message = "Unauthorized"),
            @ApiResponse(code = 403, message = "Forbidden"),
            @ApiResponse(code = 404, message = "Not Found"),
            @ApiResponse(code = 500, message = "Failure")})
    public CaseBases getCaseBases() {
        return new CaseBases();
    }

}