package no.ntnu.mycbr.rest;

import de.dfki.mycbr.core.casebase.*;
import de.dfki.mycbr.core.model.AttributeDesc;
import de.dfki.mycbr.core.model.Concept;
import de.dfki.mycbr.core.similarity.AmalgamationFct;
import de.dfki.mycbr.core.similarity.Similarity;
import de.dfki.mycbr.util.Pair;
import no.ntnu.mycbr.CBREngine;
import no.ntnu.mycbr.rest.adaptation.helpClasses.Print;
import no.ntnu.mycbr.rest.adaptation.helpClasses.Settings;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class NewRetrieval {

    public static List<Pair<Instance, Similarity>> caseBaseRetrieval(Instance desiredQuery, Instance undesiredQuery, List<Instance> instances, Concept myConcept) {
        Print.printManualRetrieval("");
        Print.printManualRetrieval("");
        Print.printManualRetrieval("");
        Print.printManualRetrieval("--------------------------------------------------------");
        Print.printManualRetrieval("--------------- Case Base Retrieval Process ------------");
        Print.printManualRetrieval("--------------------------------------------------------");

        List<Pair<Instance, Similarity>> allResults = new ArrayList<Pair<Instance, Similarity>>();
        for (Instance instance : instances) {
            Print.printManualRetrieval("");
            Print.printManualRetrieval("");
            Pair<Instance, Similarity> desiredRetrieval = NewRetrieval.retrieveSingle(desiredQuery, instance, myConcept);
            Pair<Instance, Similarity> undesiredRetrieval = NewRetrieval.retrieveSingle(undesiredQuery, instance, myConcept);

            Similarity resultingSimilarityScore = Similarity.get(desiredRetrieval.getSecond().getRoundedValue()-undesiredRetrieval.getSecond().getRoundedValue());

            Print.printManualRetrieval("%%% Desired similarity: " + desiredRetrieval.getSecond().getRoundedValue());
            Print.printManualRetrieval("%%% Undesired similarity: " + undesiredRetrieval.getSecond().getRoundedValue());
            Print.printManualRetrieval("%%% Similarity Sum: " + resultingSimilarityScore.getRoundedValue());

            allResults.add(new Pair<>(instance, resultingSimilarityScore));
        }
        List<Pair<Instance, Similarity>> selectedResults = new ArrayList<Pair<Instance, Similarity>>();

        // add instance with the best similarity to selectedResults until limit is reached (set in Settings)
        for (Pair<Instance, Similarity> res1 : allResults) {
            // finds placing for current result. If among 20 best, add to selectedResults
            int placing = 1;
            for (Pair<Instance, Similarity> res2 : allResults) {
                // if worse similarity score than res2, bump down one placing
                if (res1.getSecond().getRoundedValue() < res2.getSecond().getRoundedValue()) {
                    placing += 1;
                }
            }
            Print.printManualRetrieval(res1.getFirst().getName() + ": " + placing + " : " + res1.getSecond().getRoundedValue());
            if (placing <= Settings.numberOfRetrievedCases && selectedResults.size() < Settings.numberOfRetrievedCases) {
                selectedResults.add(res1);
            }
        }
        Print.printManualRetrieval("All results: " + allResults.size());
        Print.printManualRetrieval("Size result: " + selectedResults.size());
        return selectedResults;
    }

    public static List<Pair<Instance, Similarity>> retrieveMultiple(Instance queryInstance, List<Instance> instances, Concept myConcept) {
        Print.printManualRetrieval("");
        Print.printManualRetrieval("");
        Print.printManualRetrieval("");
        Print.printManualRetrieval("----------------------------------------------");
        Print.printManualRetrieval("--------------- Retrieval Process ------------");
        Print.printManualRetrieval("----------------------------------------------");

        List<Pair<Instance, Similarity>> results = new ArrayList<Pair<Instance, Similarity>>();
        for (Instance instance : instances) {
            Print.printManualRetrieval("");
            results.add(NewRetrieval.retrieveSingle(queryInstance, instance, myConcept));
        }
        return results;
    }

    public static Pair<Instance, Similarity> retrieveSingle(Instance queryInstance, Instance instance, Concept myConcept) {
        Print.printManualRetrieval(instance.getName());
        double similaritySum = 0.0;
        double denominator = 0.0;

        HashMap<AttributeDesc, de.dfki.mycbr.core.casebase.Attribute> attributes = instance.getAttributes();
        for (Map.Entry<AttributeDesc, de.dfki.mycbr.core.casebase.Attribute> entry : attributes.entrySet()) {
            AttributeDesc attrDesc = entry.getKey();
            String attrDescName = attrDesc.getName();
            if (!Settings.getAttributesThatShouldNotBeModified().contains(attrDescName)) {
                de.dfki.mycbr.core.casebase.Attribute queryValue = queryInstance.getAttForDesc(attrDesc);
                de.dfki.mycbr.core.casebase.Attribute instanceValue = instance.getAttForDesc(attrDesc);
                if (queryValue.getValueAsString().equals("_unknown_")) {
                    Print.printManualRetrieval(attrDesc.getName() + "; Query unknown => Ignore");
                    continue;
                }
                try {
                    Similarity sim = CBREngine.getProject().getLocalSimilarity(queryValue, instanceValue, attrDesc);
                    AmalgamationFct activeAmalgamFct = myConcept.getActiveAmalgamFct();
                    Number weight = activeAmalgamFct.getWeight(attrDesc);

                    denominator += (double) weight;
                    double attributeSimilarity = sim.getRoundedValue()*(double)weight;
                    similaritySum += attributeSimilarity;

                    Print.printManualRetrieval(attrDesc.getName() + "; " + queryValue.getValueAsString() + "(query) ; " + instanceValue.getValueAsString() + "(instance) ---> ");
                    Print.printManualRetrieval(".. " + sim.getRoundedValue() + "(sim query-instance) x " + weight + "(weight) => " + attributeSimilarity + ". SimilaritySum: " + similaritySum);
                } catch(Exception e) {
                    Print.printManualRetrieval("Error in NewRetrieval - Query");
                    e.printStackTrace();
                }
            }
        }
        Print.printManualRetrieval("Denominator: " + denominator);
        Print.printManualRetrieval("similaritySum: " + similaritySum);

        // normalizes similarity by dividing on attributes counted in the calculation
        double resultScore = similaritySum/denominator;
        Print.printManualRetrieval("similaritysum/denominator: " + resultScore);
        return new Pair<Instance, Similarity>(instance, Similarity.get(resultScore));
    }
}
