package no.ntnu.mycbr.rest;

import de.dfki.mycbr.core.DefaultCaseBase;
import de.dfki.mycbr.core.Project;
import de.dfki.mycbr.core.casebase.Attribute;
import de.dfki.mycbr.core.casebase.Instance;
import de.dfki.mycbr.core.casebase.MultipleAttribute;
import de.dfki.mycbr.core.model.*;
import de.dfki.mycbr.core.similarity.Similarity;
import de.dfki.mycbr.util.Pair;
import no.ntnu.mycbr.rest.adaptation.*;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;

public class Query {

    private static List<QueryResult> resultList;
    private String loggerFile = "./src/main/resources/query-logger.txt";

    Query(String casebase, String concept, String caseId) {
        resultList = new ArrayList<>();

        Project project = App.getProject();
        DefaultCaseBase cb = (DefaultCaseBase) project.getCaseBases().get(casebase);
        Concept myConcept = project.getConceptByID(concept);

        de.dfki.mycbr.core.retrieval.Retrieval retrieval = new de.dfki.mycbr.core.retrieval.Retrieval(myConcept, cb);
        try {
            Instance queryInstance = retrieval.getQueryInstance();
            Instance caseInstance = myConcept.getInstance(caseId);
            for (Map.Entry<AttributeDesc, Attribute> e : caseInstance.getAttributes().entrySet()) {
                queryInstance.addAttribute(e.getKey(), e.getValue());
            }
            retrieval.setRetrievalMethod(de.dfki.mycbr.core.retrieval.Retrieval.RetrievalMethod.RETRIEVE_SORTED);
            retrieval.start();
            List<Pair<Instance, Similarity>> results = retrieval.getResult();
            for (Pair<Instance, Similarity> result : results) {
                resultList.add(new QueryResult(result.getFirst().getName(), result.getSecond().getValue()));
            }
            queryInstance.reset();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    Query(String casebase, String concept, HashMap<String, HashMap<String, Object>> queryContent, int k, boolean hackToNotAdapt) {
        resultList = new ArrayList<>();

        Project project = App.getProject();
        DefaultCaseBase cb = (DefaultCaseBase) project.getCaseBases().get(casebase);
        Concept myConcept = project.getConceptByID(concept);

        HashMap<String, Object> desiredValues = queryContent.get("desired");
        HashMap<String, Object> undesiredValues = queryContent.get("undesired");

        try {
            Instance query = new Instance(myConcept, "Query-Desired");
            Instance queryU = new Instance(myConcept, "Query-Undesired");

            // Get query instances
            Instance queryDesired = this.getQueryInstance(query, myConcept, desiredValues);
            Instance queryUndesired = this.getQueryInstance(queryU, myConcept, undesiredValues);

            ArrayList<Instance> originalInstances = new ArrayList<>(cb.getCases());
            List<Pair<Instance, Similarity>> results = NewRetrieval.retrieveMultiple(queryDesired, originalInstances, myConcept);

            // modify undesired query
            queryUndesired = PreAdaptation.addChildrenOfUndesiredIngredients(queryUndesired, myConcept);

            // filter according to undesired query
            results = PostAdaptationOriginal.filterOriginalRecipes(myConcept, queryUndesired, results);

            // Add all cases to result
            for (Pair<Instance, Similarity> result : results) {
                resultList.add(new QueryResult(result.getFirst().getName(), result.getSecond().getValue()));
            }

            // Sort resultList by score, descending
            resultList.sort((o1, o2) -> o2.score.compareTo(o1.score));

            queryDesired.reset();

        } catch (Exception e) {
            e.printStackTrace();
        }

    }


    // Query that involves adaptation
    Query(String casebase, String concept, HashMap<String, HashMap<String, Object>> queryContent, int k) {
        logToFile(queryContent.toString());
        resultList = new ArrayList<>();

        Project project = App.getProject();
        DefaultCaseBase cb = (DefaultCaseBase) project.getCaseBases().get(casebase);
        Concept myConcept = project.getConceptByID(concept);

        try {
            Instance query = new Instance(myConcept, "Query-Desired");
            Instance queryU = new Instance(myConcept, "Query-Undesired");

            // Get query instances
            Instance queryDesired = this.getQueryInstance(query, myConcept, queryContent.get("desired"));
            Instance queryUndesired = this.getQueryInstance(queryU, myConcept, queryContent.get("undesired"));

            // Remove adapted cases from case base
            PreAdaptation.removeAdaptedInstancesFromCasebase(myConcept);

            // Modify undesired query
            queryUndesired = PreAdaptation.addChildrenOfUndesiredIngredients(queryUndesired, myConcept);

            // Start retrieval for original instances
            ArrayList<Instance> originalInstances = new ArrayList<>(cb.getCases());
            List<Pair<Instance, Similarity>> originalResults = NewRetrieval.caseBaseRetrieval(queryDesired, queryUndesired, originalInstances, myConcept);

            // Do adaptation on retrieved original recipes
            Adaptation a = new Adaptation(queryDesired, queryUndesired, originalResults);

            // Add adapted cases to result
            for (Pair<Instance, Similarity> result : a.getFinalAdaptedResults()) {
                List<Pair<String, String>> substitutions = a.getSubstitutionsForInstance(result.getFirst());
                List<Pair<String, String>> parentChildrenList = PostAdaptation.getParentChildrenList(queryDesired, result.getFirst());
                List<Pair<String, String>> childrenParentList = PostAdaptation.getChildrenParentList(queryDesired, result.getFirst());
                resultList.add(new QueryResult(result.getFirst().getName(), result.getSecond().getValue(), substitutions, parentChildrenList, childrenParentList));
            }

            // Add original cases to result
            for (Pair<Instance, Similarity> result : a.getFinalOriginalResults()) {
                List<Pair<String, String>> parentChildrenList = PostAdaptation.getParentChildrenList(queryDesired, result.getFirst());
                List<Pair<String, String>> childrenParentList = PostAdaptation.getChildrenParentList(queryDesired, result.getFirst());
                resultList.add(new QueryResult(result.getFirst().getName(), result.getSecond().getValue(), a.getSubstitutionsForInstance(result.getFirst()), parentChildrenList, childrenParentList));
            }

            // Sort resultList by score, descending
            resultList.sort((o1, o2) -> o2.score.compareTo(o1.score));

            queryDesired.reset();
            queryUndesired.reset();

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public List<QueryResult> getSimilarCases() {
        return resultList;
    }

    private Instance getQueryInstance(Instance query, Concept myConcept, HashMap<String, Object> attributeList) {
        if (attributeList != null) {
            try {
                for (Map.Entry<String, Object> attribute : attributeList.entrySet()) {
                    String attributeName = attribute.getKey();
                    AttributeDesc attributeDesc = myConcept.getAttributeDesc(attributeName);
                    if (attributeDesc.getClass().getSimpleName().equalsIgnoreCase("FloatDesc")) {
                        query.addAttribute(attributeDesc, Float.parseFloat(attribute.getValue().toString()));
                    }
                    if (attributeDesc.getClass().getSimpleName().equalsIgnoreCase("IntegerDesc")) {
                        query.addAttribute(attributeDesc, Integer.parseInt(attribute.getValue().toString()));
                    }
                    if (attributeDesc.getClass().getSimpleName().equalsIgnoreCase("DoubleDesc")) {
                        query.addAttribute(attributeDesc, Double.parseDouble(attribute.getValue().toString()));
                    }
                    if (attributeDesc.getClass().getSimpleName().equalsIgnoreCase("SymbolDesc")) {
                        SymbolDesc symbolAttribute = (SymbolDesc) attributeDesc;
                        if (!symbolAttribute.isMultiple()) {
                            query.addAttribute(attributeDesc, attribute.getValue());
                        } else {
                            LinkedList<Attribute> attributeLinkedList = new LinkedList<>();
                            StringTokenizer stringTokenizer = new StringTokenizer((String) attribute.getValue(), ",");
                            while (stringTokenizer.hasMoreElements()) {
                                String symbolName = stringTokenizer.nextElement().toString().trim();
                                attributeLinkedList.add(symbolAttribute.getAttribute(symbolName));
                            }
                            MultipleAttribute<SymbolDesc> multiSymbol = new MultipleAttribute<>(symbolAttribute, attributeLinkedList);
                            query.addAttribute(attributeDesc, multiSymbol);
                        }
                    }
                }
                return query;

            } catch (Exception e) {
                e.printStackTrace();
                System.out.println("Query getQueryInstance" + query);
                System.out.println("returns ERROR_INSTANCE (getQueryInstance)");
            }
        }
        return new Instance(myConcept, "ERROR-INSTANCE");
    }

    private void logToFile(String rawQuery) {
        FileWriter fw = null;
        BufferedWriter bw = null;
        PrintWriter out = null;
        try {
            fw = new FileWriter(loggerFile, true);
            bw = new BufferedWriter(fw);
            out = new PrintWriter(bw);
            out.println(rawQuery);
            out.close();
        } catch (IOException e) {
        }
        finally {
            if(out != null)
                out.close();
            try {
                if(bw != null)
                    bw.close();
            } catch (IOException e) {
            }
            try {
                if(fw != null)
                    fw.close();
            } catch (IOException e) {
            }
        }

    }
}