package no.ntnu.mycbr.rest.adaptation;

import de.dfki.mycbr.core.casebase.Instance;
import de.dfki.mycbr.core.similarity.Similarity;
import de.dfki.mycbr.util.Pair;
import no.ntnu.mycbr.rest.adaptation.adaptedInstance.AdaptationProcess;
import no.ntnu.mycbr.rest.adaptation.helpClasses.Operations;
import no.ntnu.mycbr.rest.adaptation.helpClasses.OperationsTaxonomy;
import no.ntnu.mycbr.rest.adaptation.helpClasses.Print;

import java.util.*;

import static java.util.Collections.reverseOrder;
import static java.util.Comparator.comparing;

public class PostAdaptation {

    // retuns list with parent-children relationship (for frontend usage)
    public static List<Pair<String, String>> getParentChildrenList(Instance desiredQuery, Instance originalInstance) {
        List<Pair<String, String>> parentChildrenList = new ArrayList<Pair<String, String>>();
        HashSet<String> allQueryIngredients = Operations.getAllIngredientsFromInstance(desiredQuery);
        HashSet<String> allOriginalIngredients = Operations.getAllIngredientsFromInstance(originalInstance);
        for (String queryIng : allQueryIngredients) {
            try {
                if (allOriginalIngredients.contains(queryIng)) {
                    continue;
                }
                List<String> childrenOfQueryIng = OperationsTaxonomy.getChildrenOfIngredient(queryIng);
                for (String child : childrenOfQueryIng) {
                    if (allOriginalIngredients.contains(child)) {
                        parentChildrenList.add(new Pair<String, String>(queryIng, child));
                    }
                }
            } catch (Exception e) {
                System.out.println("Error in getParentChildrenList for originalInstance");
                e.printStackTrace();
            }

        }
        return parentChildrenList;
    }
    // retuns list with children-parent relationship (for frontend usage)
    public static List<Pair<String, String>> getChildrenParentList(Instance desiredQuery, Instance originalInstance) {
        HashSet<String> allQueryIngredients = Operations.getAllIngredientsFromInstance(desiredQuery);
        HashSet<String> allOriginalIngredients = Operations.getAllIngredientsFromInstance(originalInstance);
        List<Pair<String, String>> childrenParentList = new ArrayList<Pair<String, String>>();

        for (String desiredIngredient : allQueryIngredients) {
            try {
                if (allOriginalIngredients.contains(desiredIngredient)) {
                    continue;
                }
                for (String parent : allOriginalIngredients) {
                    List<String> childrenOfParent = OperationsTaxonomy.getChildrenOfIngredient(parent);
                    for (String child : childrenOfParent) {
                        if (desiredIngredient.equals(child)) {
                            childrenParentList.add(new Pair<String, String>(desiredIngredient, parent));
                        }
                    }
                }
            } catch (Exception e) {
                System.out.println("Error in getParentChildrenList for originalInstance");
                e.printStackTrace();
            }
        }
        return childrenParentList;
    }

    // pair and print results
    public static void pairAndPrintResults(Adaptation a,
                                           List<Pair<Instance, Similarity>> originalResults,
                                           List<Pair<Instance, Similarity>> adaptedResults,
                                           List<Pair<Instance, Similarity>> punishedAdaptedResults,
                                           Instance query) {
        List<AdaptationProcess> invalidAdaptationProcesses = a.getInvalidAdaptationProcesses();
        List<AdaptationProcess> zeroAdaptationProcesses = a.getZeroAdaptationProcesses();

        Print.printInvalidProcess(invalidAdaptationProcesses);
        Print.printZeroProcess(zeroAdaptationProcesses);

        final List<Pair<Instance, Double>> pairs = new ArrayList<>();

        for (Pair<Instance, Similarity> aResult : adaptedResults) {
            Pair<Instance, Double> adaptedResult = new Pair<Instance, Double>(aResult.getFirst(), aResult.getSecond().getRoundedValue());
            pairs.add(adaptedResult);
        }

        final Comparator<Pair<Instance, Double>> score = reverseOrder(comparing(Pair::getSecond));
        Collections.sort(pairs, score);

        for (Pair<Instance, Double> adaptedResultSortedByDouble : pairs) {
            for (Pair<Instance, Similarity> aResult : adaptedResults) {
                for (Pair<Instance, Similarity> originalResult : originalResults) {
                    if (aResult.getFirst().getName().contains(originalResult.getFirst().getName() + "-") && adaptedResultSortedByDouble.getFirst().getName().contains(aResult.getFirst().getName())) {
                        for (Pair<Instance, Similarity> punishedAdaptedResult : punishedAdaptedResults) {
                            if (punishedAdaptedResult.getFirst().getName().equals(aResult.getFirst().getName())) {
                                AdaptationProcess adaptationProcess = a.getAdaptationProcessForInstance(aResult.getFirst());
                                Print.printOneProcess(originalResult, aResult, punishedAdaptedResult, adaptationProcess, query);
                                Print.printSimpleAdaptationEvaluation(originalResult, aResult, punishedAdaptedResult, adaptationProcess, query);
                            }
                        }
                    }
                }
            }
        }

        for (Pair<Instance, Double> adaptedResultSortedByDouble : pairs) {
            for (Pair<Instance, Similarity> aResult : adaptedResults) {
                for (Pair<Instance, Similarity> originalResult : originalResults) {
                    if (aResult.getFirst().getName().contains(originalResult.getFirst().getName() + "-") && adaptedResultSortedByDouble.getFirst().getName().contains(aResult.getFirst().getName())) {
                        for (Pair<Instance, Similarity> punishedAdaptedResult : punishedAdaptedResults) {
                            if (punishedAdaptedResult.getFirst().getName().equals(aResult.getFirst().getName())) {
                                AdaptationProcess adaptationProcess = a.getAdaptationProcessForInstance(aResult.getFirst());
                                Print.printDetailedAdaptationEvaluation(originalResult, aResult, punishedAdaptedResult, adaptationProcess, query);
                            }
                        }
                    }
                }
            }
        }

        Print.printGlobalStatistics(a.getGlobalStatistics());
        Print.printFinalNumbers(a.getAdaptationProcesses().size(), a.getZeroAdaptationProcesses().size(), a.getInvalidAdaptationProcesses().size());
    }
}