package no.ntnu.mycbr.rest.adaptation;

import de.dfki.mycbr.core.casebase.Instance;
import de.dfki.mycbr.core.model.Concept;
import de.dfki.mycbr.core.similarity.Similarity;
import de.dfki.mycbr.util.Pair;
import no.ntnu.mycbr.rest.NewRetrieval;
import no.ntnu.mycbr.rest.adaptation.helpClasses.Operations;
import no.ntnu.mycbr.rest.adaptation.helpClasses.Print;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class PostAdaptationAdapted {
    // Post adapatation for adapted instances

    // Removes instances from adaptedResults
    public static void removeEqualRecipes(List<Pair<Instance, Similarity>> originalResults, List<Pair<Instance, Similarity>> adaptedResults) {
        //List<Pair<Instance, Similarity>> uniqueAdaptedInstances = new ArrayList<Pair<Instance, Similarity>>();
        List<Pair<Instance, Similarity>> uniqueResults = new ArrayList<Pair<Instance, Similarity>>();
        uniqueResults.addAll(originalResults);
        Integer removedRecipes = 0;
        List<Pair<Instance, Similarity>> resultsToBeRemoved = new ArrayList<Pair<Instance, Similarity>>();

        for (Pair<Instance, Similarity> adaptedResult : adaptedResults) {
            Instance adaptedInstance = adaptedResult.getFirst();
            Boolean isUnique = true;

            // check if adapted instance is equal one of the unique instances
            for (Pair<Instance, Similarity> uniqueResult : uniqueResults) {
                Instance uniqueInstance = uniqueResult.getFirst();
                HashSet<String> allIngredientsInAdaptedInstance = Operations.getAllIngredientsFromInstance(adaptedInstance);
                HashSet<String> allIngredientsInUniqueInstance = Operations.getAllIngredientsFromInstance(uniqueInstance);

                if (listEqualsNoOrder(allIngredientsInAdaptedInstance, allIngredientsInUniqueInstance)) {
                    Print.printRemoveEqualRecipesProcess("All ingredients in adapted instance: (" +adaptedInstance.getName() + ") " + allIngredientsInAdaptedInstance);
                    Print.printRemoveEqualRecipesProcess("All ingredients in original instance: (" +uniqueInstance.getName() + ") " + allIngredientsInUniqueInstance);
                    Print.printRemoveEqualRecipesProcess(adaptedInstance.getName() + " is not unique (same ingredients as " + uniqueInstance.getName() + ") --> Removed from resultList.");
                    isUnique = false;
                    removedRecipes += 1;
                }
            }
            if (isUnique) {
                uniqueResults.add(adaptedResult);
                //uniqueAdaptedInstances.add(adaptedResult);
            }
            else {
                resultsToBeRemoved.add(adaptedResult);
            }
        }

        for (Pair<Instance, Similarity> resultToBeRemoved : resultsToBeRemoved) {
            adaptedResults.remove(resultToBeRemoved);
        }

        Print.printRemoveEqualRecipesProcess("");
        Print.printRemoveEqualRecipesProcess(removedRecipes + " recipes are removed because some were equal (contained same ingredients)");
        Print.printRemoveEqualRecipesProcess("");
        //return uniqueAdaptedInstances;
    }

    public static <T> boolean listEqualsNoOrder(HashSet<T> l1, HashSet<T> l2) {
        final Set<T> s1 = new HashSet<>(l1);
        final Set<T> s2 = new HashSet<>(l2);

        return s1.equals(s2);
    }

    // punish adapted instances
    public static List<Pair<Instance, Similarity>> punishAdaptedInstances(List<Pair<Instance, Similarity>> originalResults, List<Pair<Instance, Similarity>> adaptedResults, Concept myConcept, Adaptation a) {
        Print.printPunishAdaptedRecipeProcess("");
        Print.printPunishAdaptedRecipeProcess("");
        Print.printPunishAdaptedRecipeProcess("----------------------------------------------");
        Print.printPunishAdaptedRecipeProcess("------- Punish adapted recipes process -------");
        Print.printPunishAdaptedRecipeProcess("----------------------------------------------");
        Print.printPunishAdaptedRecipeProcess("");

        List<Pair<Instance, Similarity>> punishedAdaptedResults = new ArrayList<Pair<Instance, Similarity>>();

        for (Pair<Instance, Similarity> adaptedResult : adaptedResults) {
            for (Pair<Instance, Similarity> originalResult : originalResults) {
                Instance originalInstance = originalResult.getFirst();
                Instance adaptedInstance = adaptedResult.getFirst();
                // Pairs original and adapted instance
                if (adaptedResult.getFirst().getName().contains(originalResult.getFirst().getName() + "-")) {
                    Print.printPunishAdaptedRecipeProcess("");
                    Print.printPunishAdaptedRecipeProcess("Punish " + adaptedResult.getFirst().getName());

                    double previousSimilarity = adaptedResult.getSecond().getRoundedValue();
                    double punishment = NewRetrieval.retrieveSingle(adaptedInstance, originalInstance, myConcept).getSecond().getRoundedValue();
                    double result = previousSimilarity*punishment;

                    Print.printPunishAdaptedRecipeProcess("");
                    Print.printPunishAdaptedRecipeProcess("Similarity query-adapted:   " + previousSimilarity);
                    Print.printPunishAdaptedRecipeProcess("Similarity adapted-original: " + punishment);
                    Print.printPunishAdaptedRecipeProcess("Similarity result:           " + result);

                    // copy adapted instance sets new similarity score
                    Instance newInstance = adaptedResult.getFirst();
                    Similarity newSimilarity = Similarity.get(result);
                    Pair<Instance, Similarity> newPair = new Pair<Instance, Similarity>(newInstance, newSimilarity);
                    punishedAdaptedResults.add(newPair);
                }
            }
        }
        return punishedAdaptedResults;
    }
}
