package no.ntnu.mycbr.rest.adaptation;

import de.dfki.mycbr.core.casebase.Attribute;
import de.dfki.mycbr.core.casebase.Instance;
import de.dfki.mycbr.core.casebase.MultipleAttribute;
import de.dfki.mycbr.core.model.AttributeDesc;
import de.dfki.mycbr.core.model.Concept;
import de.dfki.mycbr.core.model.SymbolDesc;
import de.dfki.mycbr.core.similarity.AmalgamationFct;
import de.dfki.mycbr.core.similarity.Taxonomy;
import de.dfki.mycbr.core.similarity.TaxonomyFct;
import de.dfki.mycbr.core.similarity.TaxonomyNode;
import no.ntnu.mycbr.rest.adaptation.helpClasses.Operations;
import no.ntnu.mycbr.rest.adaptation.helpClasses.Print;
import no.ntnu.mycbr.rest.adaptation.helpClasses.Settings;

import java.util.*;

public class PreAdaptation {

    public static void removeAdaptedInstancesFromCasebase(Concept myConcept) {
        // Find adapted instances
        Collection<Instance> allInstances = myConcept.getAllInstances();
        List<Instance> removeInstances = new ArrayList<Instance>();
        for (Instance inst : allInstances) {
            if (inst.getName().contains("-Adapted") || (inst.getName().contains("-ModifiedByUndesired"))) {
                removeInstances.add(inst);
            }
        }
        // remove adapted instances from concept
        for (Instance inst : removeInstances) {
            myConcept.removeInstance(inst.getName());
        }
    }

    public static Instance addChildrenOfUndesiredIngredients(Instance queryInstance, Concept myConcept) throws Exception {
        Print.printUndesiredQueryProcess("");
        Print.printUndesiredQueryProcess("");

        Print.printUndesiredQueryProcess("--------------------------------------");
        Print.printUndesiredQueryProcess("------- Add undesired children -------");
        Print.printUndesiredQueryProcess("--------------------------------------");

        // For each attributeDesc: Fetch children from similarity hierarchy
        HashMap<String, AttributeDesc> allAttributeDescs = myConcept.getAllAttributeDescs();
        List<AmalgamationFct> availableAmalgamFcts = myConcept.getAvailableAmalgamFcts();

        for (Map.Entry<String, AttributeDesc> attributeDesc : allAttributeDescs.entrySet()) {
            AttributeDesc attrDesc = attributeDesc.getValue();
            Attribute attribute = queryInstance.getAttForDesc(attrDesc);

            // If requirement on attributeDesc;
            if (Operations.attributeContainsValues(attribute)) {

                List<String> attributeValuesAsList = Operations.getAttributeAsSingleValues(attribute);
                Print.printUndesiredQueryProcess(" ");
                Print.printUndesiredQueryProcess(" ");
                Print.printUndesiredQueryProcess("--------- " + attrDesc.getName() + " ---------");
                Print.printUndesiredQueryProcess("There are " + availableAmalgamFcts.size() + " availableAmalgamFcts");

                // If attribute should be considered
                if (!Settings.getTaxonomiesThatShouldNotBeConsidered().contains(attrDesc.getName())) {

                    // Create resultList for ingredients that should belong to this attribute
                    List<String> resultList = new ArrayList<String>();
                    Print.printUndesiredQueryProcess("new resultlist");

                    // Loop through each undesired ingredient on current attributeDesc
                    for (String undesiredIngredient : attributeValuesAsList) {
                        Print.printUndesiredQueryProcess("ingredient considered: " + undesiredIngredient);

                        // Loop through each amalgamationFct (Taxonomies for attributeDescs)
                        for (AmalgamationFct amFct : availableAmalgamFcts) {
                            Print.printUndesiredQueryProcess(" ");
                            Print.printUndesiredQueryProcess("Name: " + amFct.getActiveFct(attrDesc).toString());
                            Print.printUndesiredQueryProcess("AttrDesc: " + attrDesc.getName());
                            Print.printUndesiredQueryProcess("Class: " + amFct.getActiveFct(attrDesc).getClass());

                            try {
                                if (amFct.getActiveFct(attrDesc).getClass().equals(TaxonomyFct.class)) {
                                    Print.printUndesiredQueryProcess("is taxFct");
                                    // Get taxonomy for current attrDesc
                                    TaxonomyFct activeFct = ((TaxonomyFct) amFct.getActiveFct(attrDesc));

                                    Taxonomy taxonomy = activeFct.getTaxonomy();
                                    Print.printUndesiredQueryProcess("--- " + activeFct.getName() + " ---");
                                    Print.printUndesiredQueryProcess("Top symbol: " + activeFct.getTopSymbol());

                                    // Get nodes in that taxonomy
                                    List<TaxonomyNode> nodes = activeFct.getTopSymbol().getNodes();

                                    // Loop through nodes to find current undesired ingredient
                                    for (TaxonomyNode node : nodes) {
                                        if (node.toString().equals(undesiredIngredient)) {
                                            Print.printUndesiredQueryProcess(" ");
                                            Print.printUndesiredQueryProcess("MATCH");

                                            List<TaxonomyNode> allChildren = PreAdaptation.getAllChildren(node, taxonomy);
                                            // add parent and its children to result list
                                            for (TaxonomyNode child : allChildren) {
                                                resultList.add(child.toString());
                                            }

                                            // print match
                                            if (!taxonomy.getLeaves().contains(node)) {
                                                Print.printUndesiredQueryProcess(node.toString() + ", children: " + taxonomy.getChildren(node));
                                                Print.printUndesiredQueryProcess("Size: " + resultList.size());
                                            } else {
                                                Print.printUndesiredQueryProcess(node.toString());
                                            }
                                        }
                                    }
                                }
                            } catch (Exception e) {
                                //e.printStackTrace();
                                System.out.println(e);
                                System.out.println("Catched error in PreAdaptation");
                            }
                        }
                    }
                    // Add multiple attribute to current attrDesc, containing both parents and their children
                    MultipleAttribute multi = Operations.getMultipleAttribute(Operations.getListAsString(resultList), attrDesc);
                    if (multi != null) {
                        SymbolDesc symbDesc = (SymbolDesc) attrDesc;
                        queryInstance.addAttribute(symbDesc, multi);
                    }
                }
            }
        }
        return queryInstance;
    }

    // returns all its children, plus itself in a TaxonomyNode list
    public static List<TaxonomyNode> getAllChildren(TaxonomyNode currentNode, Taxonomy taxonomy) {
        List<TaxonomyNode> result = new ArrayList<TaxonomyNode>();

        List<TaxonomyNode> children = taxonomy.getChildren(currentNode);
        for (TaxonomyNode child : children) {
            List<TaxonomyNode> childsChildren = getAllChildren(child, taxonomy);
            result.addAll(childsChildren);
        }
        result.add(currentNode);
        return result;
    }

}