package no.ntnu.mycbr.rest.adaptation.adaptedInstance;

import de.dfki.mycbr.core.casebase.Instance;
import no.ntnu.mycbr.rest.adaptation.helpClasses.Settings;
import java.util.ArrayList;
import java.util.List;

public class AdaptationHistory {
    // Class made for the purpose of printing the adaptation process of an instance

    public Instance originalInstance;
    public Instance adaptedInstance;
    public List<String> history;

    private int undesiredRemoveRuleCount = 0;
    private int undesiredSubstitutionRuleCount = 0;
    private int undesiredSimilaritySubstitutionCount = 0;

    private int desiredAdditionRuleCount = 0;
    private int desiredSubstitutionRuleCount = 0;
    private int desiredSimilaritySubstitutionCount = 0;

    private int suitableIngredientsCount = 0;

    // setters undesired
    public void setUndesiredRemoveRuleCount(int i) { undesiredRemoveRuleCount = i; }
    public void setUndesiredSubstitutionRuleCount(int i) { undesiredSubstitutionRuleCount = i; }
    public void setUndesiredSimilaritySubstitutionCount(int i) { undesiredSimilaritySubstitutionCount = i; }

    // setters desired
    public void setDesiredAdditionRuleCount(int i) { desiredAdditionRuleCount = i; }
    public void setDesiredSubstitutionRuleCount(int i) { desiredSubstitutionRuleCount = i; }
    public void setDesiredSimilaritySubstitutionCount(int i) { desiredSimilaritySubstitutionCount = i; }

    // add undesired
    public void addUndesiredRemoveRuleCount() { undesiredRemoveRuleCount += 1; }
    public void addUndesiredSubstitutionRuleCount() { undesiredSubstitutionRuleCount += 1; }
    public void addUndesiredSimilaritySubstitutionCount() { undesiredSimilaritySubstitutionCount += 1; }

    // add desired
    public void addDesiredAdditionRuleCount() { desiredAdditionRuleCount += 1; }
    public void addDesiredSubstitutionRuleCount() { desiredSubstitutionRuleCount += 1; }
    public void addDesiredSimilaritySubstitutionCount() { desiredSimilaritySubstitutionCount += 1; }

    // getters undesired
    public int getUndesiredRemoveRuleCount() { return undesiredRemoveRuleCount; }
    public int getUndesiredSubstitutionRuleCount() { return undesiredSubstitutionRuleCount; }
    public int getUndesiredSimilaritySubstitutionCount() { return undesiredSimilaritySubstitutionCount; }

    // getters desired
    public int getDesiredAdditionRuleCount() { return desiredAdditionRuleCount; }
    public int getDesiredSubstitutionRuleCount() { return desiredSubstitutionRuleCount; }
    public int getDesiredSimilaritySubstitutionCount() { return desiredSimilaritySubstitutionCount; }

    // Suitable ingredients
    public void setSuitableIngredientsCount(int i) { suitableIngredientsCount = i; }
    public void addSuitableIngredientsCount() { suitableIngredientsCount += 1; }
    public int getSuitableIngredientsCount() { return suitableIngredientsCount; }

    public AdaptationHistory() {
        this.history = new ArrayList<String>();
    }

    public void addLine(String historyLine) {
        history.add(historyLine);
    }

    public AdaptationHistory(Instance oInstance, Instance aInstance) {
        this.originalInstance = oInstance;
        this.adaptedInstance = aInstance;
        this.history = new ArrayList<String>();
    }

    // is zero-process
    public Boolean undesiredChanges() {
        if (undesiredRemoveRuleCount == 0 && undesiredSubstitutionRuleCount == 0 && undesiredSimilaritySubstitutionCount == 0) {
            return false;
        }
        return true;
    }

    public Boolean desiredChanges() {
        if (desiredAdditionRuleCount == 0 && desiredSubstitutionRuleCount == 0 && desiredSimilaritySubstitutionCount == 0) {
            return false;
        }
        return true;
    }

    public int getNumberOfChanges() {
        return (undesiredRemoveRuleCount+undesiredSubstitutionRuleCount+undesiredSimilaritySubstitutionCount+
                desiredAdditionRuleCount+desiredSubstitutionRuleCount+desiredSimilaritySubstitutionCount);
    }

    public int getAdaptationPoints() {
        int points = 0;
        points += desiredAdditionRuleCount * Settings.additionRulePoints;
        points += desiredSubstitutionRuleCount * Settings.substitutionRulePoints;
        points += desiredSimilaritySubstitutionCount * Settings.similaritySubstitutionPoints;
        points += undesiredRemoveRuleCount * Settings.removalRulePoints;
        points += undesiredSubstitutionRuleCount * Settings.substitutionRulePoints;
        points += undesiredSimilaritySubstitutionCount * Settings.similaritySubstitutionPoints;
        return points;
    }
}
