package no.ntnu.mycbr.rest.adaptation.adaptedInstance;

import de.dfki.mycbr.core.casebase.Attribute;
import de.dfki.mycbr.core.model.AttributeDesc;
import de.dfki.mycbr.core.similarity.Similarity;
import de.dfki.mycbr.util.Pair;
import no.ntnu.mycbr.rest.adaptation.Adaptation;
import no.ntnu.mycbr.rest.adaptation.helpClasses.Operations;
import de.dfki.mycbr.core.casebase.Instance;
import no.ntnu.mycbr.CBREngine;
import no.ntnu.mycbr.rest.adaptation.rules.AdaptationSubstitution;
import no.ntnu.mycbr.rest.adaptation.rules.SimpleRule;
import no.ntnu.mycbr.rest.adaptation.helpClasses.SubstitutionMethods;


import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Collections;

public class SuitableAdaptation {
    // Adaptation to further fulfill the change of style for an instance
    // Considering substitutions made in desired and undesired adaptations

    public static void doSuitableAdaptation(Adaptation a, List<Pair<Instance, Similarity>> originalResults, List<Pair<Instance, Similarity>> punishedAdaptedResults, Instance qDesired, Instance qUndesired) {
        for (Pair<Instance, Similarity> originalResult : originalResults) {
            for (Pair<Instance, Similarity> adaptedResult : punishedAdaptedResults) {
                if (adaptedResult.getFirst().getName().contains(originalResult.getFirst().getName()+"-")) {
                    // Do suitable adaptation
                    AdaptationProcess adaptationProcess = a.getAdaptationProcessForInstance(adaptedResult.getFirst());
                    Instance adaptedInstance = adaptationProcess.getAdaptedInstance();
                    AdaptationHistory history = adaptationProcess.getHistory();
                    List<AdaptationSubstitution> adaptedSubstitutions = adaptationProcess.getSubstitutions();
                    List<AdaptationSubstitution> suitableSubstitutions = SuitableAdaptation.considerSuitableIngredients(history, adaptedSubstitutions, adaptedInstance, qUndesired, qDesired);
                    // add substitutions to adaptationProcess
                    a.addSuitableSubstitutions(adaptedInstance,suitableSubstitutions);
                }
            }
        }
    }

    // Goes through each ingredient that is substituted in, and tries finding suitable ingredients
    public static List<AdaptationSubstitution> considerSuitableIngredients(AdaptationHistory history,
                                                                           List<AdaptationSubstitution> adaptationSubstitutions,
                                                                           Instance adaptedInstance,
                                                                           Instance queryUndesired,
                                                                           Instance queryDesired) {

        List<AdaptationSubstitution> newSubstitutions = new ArrayList<AdaptationSubstitution>();
        try {
            history.addLine("");
            history.addLine("- Consider suitable ingredients");

            // For all ingredients that are substiuted in:
            for (AdaptationSubstitution subst : adaptationSubstitutions) {
                String newIngredient = subst.getIngredient2();
                history.addLine("- Considers new ingredient " + newIngredient);

                // Check that there is an ingredient subsituted in on that substitution
                if (newIngredient.length()>2 && !newIngredient.equals("X")) {
                    List<String> suitableIngredients = SuitableAdaptation.findSuitableIngredients(subst.getIngredient2(), adaptedInstance, queryUndesired, history);

                    if ((suitableIngredients != null) && (suitableIngredients.size() != 0)) {
                        for (String suitableIngredient : suitableIngredients) {
                            AdaptationSubstitution newSubstitution = SuitableAdaptation.considerSubstitutionMethodsForSuitableIngredient(suitableIngredient, adaptedInstance, queryDesired, history);
                            if (newSubstitution != null) {
                                newSubstitutions.add(newSubstitution);
                                break;
                            }
                        }
                    }
                    else {
                        history.addLine(".. No suitable ingredient found");
                    }
                }
            }

        } catch(Exception e) {
            System.out.println("Error in consider suitable ingredients");
            e.printStackTrace();
        }
        adaptationSubstitutions.addAll(newSubstitutions);
        return newSubstitutions;
    }

    public static List<String> findSuitableIngredients(String ingredientThatIsSubstitutedIn,
                                                       Instance adaptedInstance,
                                                       Instance queryUndesired,
                                                       AdaptationHistory history) {
        HashSet<String> allIngredients = Operations.getAllIngredientsFromInstance(adaptedInstance);
        HashSet<String> allUndesiredIngredients = Operations.getAllIngredientsFromInstance(queryUndesired);

        List<String> suitableIngredients = new ArrayList<String>();

        // Not undesired ingredient, not subst.ingredient2, not already in recipe
        for (SimpleRule suitableRule : CBREngine.getRules().suitableRules) {

            //suitableRule.printRule(history, allIngredients, ingredientThatIsSubstitutedIn);

            String suitableIngredient = suitableRule.suitableIngredientRequirementsAreSatisfied(
                    allIngredients,
                    ingredientThatIsSubstitutedIn
            );
            if (suitableIngredient != null) {
                if (allIngredients.contains(suitableIngredient)) {
                    history.addLine(".. A suitable ingredient was found (" + suitableIngredient + "), but is already in recipe so does no substitution, returns null. ");
                    return null;
                }
                else if (!allUndesiredIngredients.contains(suitableIngredient)) {
                    suitableIngredients.add(suitableIngredient);
                }
                else {
                    history.addLine(".. A suitable ingredient was found, but is also an undesired ingredient: " + suitableIngredient);
                }
            }
            else {
                history.addLine(".... no");
            }
        }

        // Shuffle list before return, so that if several suitable ingredients are valid, random is chosen!
        Collections.shuffle(suitableIngredients);
        history.addLine(".. Returns shuffeled suitable ingredients" + suitableIngredients.toString());
        return suitableIngredients;
    }

    public static AdaptationSubstitution considerSubstitutionMethodsForSuitableIngredient(String suitableIngredient,
                                                                                          Instance adaptedInstance,
                                                                                          Instance queryDesired,
                                                                                          AdaptationHistory history) {
        // Considers suitable ingredients based on current new ingredient
        AttributeDesc attributeDesc = Operations.findAttrDescForValue(suitableIngredient);
        if (attributeDesc != null) {
            Attribute attribute = adaptedInstance.getAttForDesc(attributeDesc);
            List<String> resultValues = Operations.getAttributeAsSingleValues(attribute);

            // Updates current ingredients by adding ingredients of all other attributes, and this current updated attribute
            HashSet<String> allCurrentIngredients = Operations.updateCurrentIngredients(attributeDesc, resultValues, adaptedInstance);
            Attribute queryAttribute = queryDesired.getAttForDesc(attributeDesc);
            AdaptationSubstitution newSubstitution;

            // consider simply adding rules
            newSubstitution = SubstitutionMethods.considerSimpleAdditionRules(resultValues, suitableIngredient, allCurrentIngredients, history);
            if (newSubstitution != null) {
                String resultValuesAsString = Operations.getListAsString(resultValues);
                Operations.modifyAttrDesc(attributeDesc, resultValuesAsString, history, adaptedInstance);
                return newSubstitution;
            }
            // consider substitution rules
            newSubstitution = SubstitutionMethods.considerDesiredSubstitutionRules(queryAttribute, resultValues, suitableIngredient, allCurrentIngredients, history);
            if (newSubstitution != null) {
                String resultValuesAsString = Operations.getListAsString(resultValues);
                Operations.modifyAttrDesc(attributeDesc, resultValuesAsString, history, adaptedInstance);
                // update offer attribute
                String offer = newSubstitution.getIngredient1();
                AttributeDesc offerAttrDesc = Operations.findAttrDescForValue(offer);
                Attribute offerAttribute = adaptedInstance.getAttForDesc(offerAttrDesc);
                List<String> offerAttributeValues = Operations.getAttributeAsSingleValues(offerAttribute);
                offerAttributeValues.remove(offer);
                String ModifiedAttributeOfferAsString = Operations.getListAsString(offerAttributeValues);
                Operations.modifyAttrDesc(offerAttrDesc, ModifiedAttributeOfferAsString, history, adaptedInstance);

                return newSubstitution;
            }

            // Use similarity substitution to find an ingredient to substitute out
            try {
                newSubstitution = SubstitutionMethods.considerSimilaritySubstituion(attributeDesc, suitableIngredient, resultValues, queryDesired, history);
                if (newSubstitution != null) {
                    String resultValuesAsString = Operations.getListAsString(resultValues);
                    Operations.modifyAttrDesc(attributeDesc, resultValuesAsString, history, adaptedInstance);
                    return newSubstitution;
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
            history.addLine(".. Suitable ingredient " + suitableIngredient + " is not satisfied, continues.");
        }
        else {
            history.addLine(".. Could not find attrDesc for " + suitableIngredient);
        }
        return null;
    }
}
