package no.ntnu.mycbr.rest.adaptation.adaptedInstance;

import de.dfki.mycbr.core.casebase.Instance;
import de.dfki.mycbr.core.model.AttributeDesc;
import de.dfki.mycbr.core.model.Concept;
import de.dfki.mycbr.core.model.StringDesc;
import de.dfki.mycbr.core.similarity.Similarity;
import de.dfki.mycbr.util.Pair;
import no.ntnu.mycbr.CBREngine;
import no.ntnu.mycbr.rest.adaptation.Adaptation;
import no.ntnu.mycbr.rest.adaptation.helpClasses.Operations;
import no.ntnu.mycbr.rest.adaptation.rules.AdaptationSubstitution;
import no.ntnu.mycbr.rest.adaptation.rules.SimpleRule;

import java.text.ParseException;
import java.util.HashSet;
import java.util.List;

public class TitleAdaptation {
    // Renames titles if titleRules are found

    public static void doTitleAdaptation(Adaptation a, List<Pair<Instance, Similarity>> originalResults, List<Pair<Instance, Similarity>> punishedAdaptedResults, Instance qDesired, Instance qUndesired) {

        for (Pair<Instance, Similarity> originalResult : originalResults) {
            for (Pair<Instance, Similarity> adaptedResult : punishedAdaptedResults) {
                if (adaptedResult.getFirst().getName().contains(originalResult.getFirst().getName()+"-")) {
                    // Do title adaptation
                    AdaptationProcess adaptationProcess = a.getAdaptationProcessForInstance(adaptedResult.getFirst());
                    Instance adaptedInstance = adaptationProcess.getAdaptedInstance();
                    Instance originalInstance = originalResult.getFirst();
                    AdaptationHistory history = adaptationProcess.getHistory();
                    List<AdaptationSubstitution> adaptedSubstitutions = adaptationProcess.getSubstitutions();
                    try {
                        TitleAdaptation.autogenerateTitle(history, CBREngine.getConcept(), adaptedInstance, originalInstance, adaptedSubstitutions);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }

    public static void autogenerateTitle(AdaptationHistory history,
                                         Concept myConcept, Instance adaptedInstance,
                                         Instance originalInstance,
                                         List<AdaptationSubstitution> adaptationSubstitutions) throws ParseException {
        history.addLine("");
        history.addLine("- Tries autogenerating title");
        AttributeDesc attrDesc = myConcept.getAttributeDesc("Title");
        String title = adaptedInstance.getAttForDesc(attrDesc).getValueAsString().toLowerCase();
        Boolean isModified = false;
        HashSet<String> allIngredients = Operations.getAllIngredientsFromInstance(adaptedInstance);
        title = TitleAdaptation.getSpecifiedTitle(title, allIngredients, adaptedInstance, history, adaptationSubstitutions);

        title = title.substring(0, 1).toUpperCase() + title.substring(1);
        if (title.contains("b.l.t")) {
            title = title.replace("b.l.t", "B.L.T");
        }

        try {
            StringDesc titleDesc = (StringDesc) attrDesc;
            adaptedInstance.addAttribute(titleDesc, titleDesc.getAttribute(title));
        } catch (Exception e) {
            e.printStackTrace();
            history.addLine("Error. Modifying title");
        }

        if (isModified) {
            history.addLine("Old attribute: " + originalInstance.getAttForDesc(attrDesc));
            history.addLine("New attribute: " + adaptedInstance.getAttForDesc(attrDesc));
        }

        history.addLine(".. Attribute title is set as: " + adaptedInstance.getAttForDesc(attrDesc).getValueAsString());
    }

    // Applies to titles containing an alternative to model ingredient
    public static String getAlternativeTitleMatch(String ingredient) {
        switch (ingredient) {
            case "smoked salmon": return "salmon";
            case "crabmeat": return "crab";
            case "pita bread": return "pita";
            case "chicken breast": return "chicken";
            case "roast turkey": return "turkey";
        }
        return null;
    }

    public static String getSpecifiedTitle(String title, HashSet<String> allIngredients, Instance adaptedInstance, AdaptationHistory history, List<AdaptationSubstitution> adaptationSubstitutions) {
        for (SimpleRule titleRule: CBREngine.getRules().titleRules) {
            if (titleRule.getTitleRule(allIngredients) != null) {
//                history.addLine("Returns rule for title" + title);
                return titleRule.resultIngredients.get(0);
            }
        }
        // Previous title adapatation
        if (title.contains("")) {
            for (AdaptationSubstitution subst : adaptationSubstitutions) {
                String ing1 = subst.getIngredient1();
                String ing1Alternative = TitleAdaptation.getAlternativeTitleMatch(ing1);

                if (ing1Alternative != null && title.contains(ing1Alternative)) {
                    title = title.replace(ing1Alternative, subst.getIngredient2());
                }

                else if (title.contains(subst.getIngredient1())) {
                    title = title.replace(subst.getIngredient1(), subst.getIngredient2());
                }
            }
        }
        return title;
    }
}
