package no.ntnu.mycbr.rest.adaptation.helpClasses;

import de.dfki.mycbr.core.casebase.Instance;
import de.dfki.mycbr.core.casebase.Attribute;
import de.dfki.mycbr.core.casebase.MultipleAttribute;
import de.dfki.mycbr.core.model.AttributeDesc;
import de.dfki.mycbr.core.model.SymbolDesc;
import de.dfki.mycbr.core.model.Concept;
import no.ntnu.mycbr.CBREngine;
import no.ntnu.mycbr.rest.adaptation.adaptedInstance.AdaptationHistory;

import java.util.*;

public class Operations {

    // returns true if the value exist in the myCBR model
    public static boolean ingredientExistInModel(String ingredient) {
        AttributeDesc attrDescForValue = Operations.findAttrDescForValue(ingredient);
        if (attrDescForValue != null) {
            return true;
        }
        return false;
    }

    // finds corresponding attribute desc for an ingredient value
    public static AttributeDesc findAttrDescForValue(String value) {
        try {
            for (Map.Entry<String, AttributeDesc> entry : CBREngine.getConcept().getAttributeDescs().entrySet()) {
                AttributeDesc attrDesc = entry.getValue();
                if (attrDesc instanceof SymbolDesc) {
                    SymbolDesc aSymbolAtt = (SymbolDesc) attrDesc;
                    Set<String> allowedValues = aSymbolAtt.getAllowedValues();
                    if (allowedValues.contains(value)) {
                        return attrDesc;
                    }
                }
            }
        }
        catch(Exception e) {
            System.out.println("Operations: Error returning attrDesc for value");
        }
        return null;
    }

    // transfers a list to a simple string
    public static String getListAsString(List<String> list) {
        String result = "";
        for (String ingredient : list) {
            if (Operations.ingredientIsValid(ingredient)) {
                result += ingredient;
                result += ",";
            }
        }
        // Strip last comma
        if (result.length() > 1) {
            result = result.substring(0, result.length()-1);
        }
        return result;
    }

    // returns a list containing the values in the attribute
    public static List<String> getAttributeAsSingleValues(Attribute attribute) {
        List<String> ingredientList = new ArrayList<String>();
        String[] values = attribute.getValueAsString().split(";", 200);
        for (String ingredient : values) {
            if (Operations.ingredientIsValid(ingredient)) {ingredientList.add(ingredient);}
        }
        return ingredientList;
    }

    // returns true if the ingredient is valid
    public static Boolean ingredientIsValid(String ingredient) {
        return (!ingredient.equals("_unknown_") && !ingredient.equals("_undefined_") && ingredient.length() > 2);
    }

    // returns true if the attribute is not unknown or undefined
    public static Boolean attributeContainsValues(Attribute attribute) {
        return (!attribute.getValueAsString().equals("_undefined_") && !attribute.getValueAsString().equals("_unknown_") && attribute.getValueAsString().length() > 2);
    }

    // returns mulitpleAttribute containing the values enlisted
    public static MultipleAttribute<SymbolDesc> getMultipleAttribute(String ingredients, AttributeDesc attrDesc) {
        try {
            SymbolDesc symbDesc = (SymbolDesc) attrDesc;
            if (!Operations.ingredientIsValid(ingredients)) {
                return null;
            }
            LinkedList<Attribute> linkedListAttribute = new LinkedList<Attribute>();
            StringTokenizer st = new StringTokenizer(ingredients, ",");

            while (st.hasMoreElements()) {
                String symbolName = st.nextElement().toString().trim();
                linkedListAttribute.add(symbDesc.getAttribute(symbolName));
            }
            MultipleAttribute<SymbolDesc> multiSymbol = new MultipleAttribute<SymbolDesc>(symbDesc, linkedListAttribute);

            return multiSymbol;

        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    // returns a hashSet of ingredients in an instance
    public static HashSet<String> getAllIngredientsFromInstance(Instance instance) {
        HashSet<String> allIngredients = new HashSet<>();

        if (instance==null) {
            return allIngredients;
        }

        Concept myConcept = CBREngine.getConcept();
        for (Map.Entry<String, AttributeDesc> entry : myConcept.getAttributeDescs().entrySet()) {
            AttributeDesc attrDesc = entry.getValue();
            if (!Settings.getAttributesThatShouldNotBeModified().contains(attrDesc.getName())) {
                Attribute instanceAttribute = instance.getAttForDesc(attrDesc);
                List<String> instanceAttributeAsList = Operations.getAttributeAsSingleValues(instanceAttribute);
                allIngredients.addAll(instanceAttributeAsList);
            }
        }
        return allIngredients;
    }

    // modifies current attribute based on the new modifiedIngredients string
    public static void modifyAttrDesc(AttributeDesc attrDesc, String modifiedIngredients, AdaptationHistory history, Instance adaptedInstance) {
        try {
            SymbolDesc symbDesc = (SymbolDesc) attrDesc;
            if (!Operations.ingredientIsValid(modifiedIngredients)) {

                history.addLine("..The modifiedIngredients result is empty - It means that it was previously unknown. OR that ingredients that were there are removed.");
                history.addLine("..No rules were applied. Keeps old value.");
                // set attrDesc to unknown
                adaptedInstance.addAttribute(attrDesc, "_unknown_");
                history.addLine("added attribute unknown. Now is: " + adaptedInstance.getAttForDesc(attrDesc));
                return;
            }
            LinkedList<Attribute> linkedListAttribute = new LinkedList<Attribute>();
            StringTokenizer st = new StringTokenizer(modifiedIngredients, ",");

            while (st.hasMoreElements()) {
                String symbolName = st.nextElement().toString().trim();
                linkedListAttribute.add(symbDesc.getAttribute(symbolName));
            }
            MultipleAttribute<SymbolDesc> multiSymbol = new MultipleAttribute<SymbolDesc>(symbDesc, linkedListAttribute);
            adaptedInstance.addAttribute(symbDesc, multiSymbol);
            history.addLine(attrDesc.getName() + ": " + adaptedInstance.getAttForDesc(attrDesc));
            return;

        } catch (Exception e) {
            e.printStackTrace();
            history.addLine("Error. Is not a SymbolDesc Attribute. Do not modify it or create new method to modify this attribute type");
        }
        history.addLine(".. Ingredient is required, but no modification is made. Error.");
    }


    // updates current ingredients and returns the updated list
    public static HashSet<String> updateCurrentIngredients(AttributeDesc currentlyUpdatingAttrDesc, List<String> currentValuesForUpdatingAttrDesc, Instance adaptedInstance) {
        HashSet<String> currentIngredients = new HashSet<String>();

        // Add ingredients that are on all other attrDescs than the one that is currently being updated
        for (Map.Entry<String, AttributeDesc> entry : CBREngine.getConcept().getAttributeDescs().entrySet()) {
            AttributeDesc attrDesc = entry.getValue();
            if (!attrDesc.getName().equals(currentlyUpdatingAttrDesc) && !Settings.getAttributesThatShouldNotBeModified().contains(attrDesc.getName())) {
                List<String> ingredients = Operations.getAttributeAsSingleValues(adaptedInstance.getAttForDesc(attrDesc));
                currentIngredients.addAll(ingredients);
            }
        }
        // Add ingredients on the attrDesc that is currently being updated
        currentIngredients.addAll(currentValuesForUpdatingAttrDesc);
        return currentIngredients;
    }
}
