package no.ntnu.mycbr.rest.adaptation.helpClasses;

import de.dfki.mycbr.core.model.AttributeDesc;
import de.dfki.mycbr.core.similarity.AmalgamationFct;
import de.dfki.mycbr.core.similarity.Taxonomy;
import de.dfki.mycbr.core.similarity.TaxonomyFct;
import de.dfki.mycbr.core.similarity.TaxonomyNode;
import no.ntnu.mycbr.CBREngine;
import no.ntnu.mycbr.rest.adaptation.PreAdaptation;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

public class OperationsTaxonomy {

    // returns all children of the ingredient
    public static List<String> getChildrenOfIngredient(String ingredient) throws Exception {
        List<String> resultList = new ArrayList<String>();
        try {
            AttributeDesc attrDesc = Operations.findAttrDescForValue(ingredient);
            List<AmalgamationFct> availableAmalgamFcts = CBREngine.getConcept().getAvailableAmalgamFcts();
            if (attrDesc != null) {
                // Loop through each amalgamationFct (Taxonomies for attributeDescs)
                for (AmalgamationFct amFct : availableAmalgamFcts) {
                    if (amFct.getActiveFct(attrDesc).getClass().equals(TaxonomyFct.class)) {
                        // Get taxonomy for current attrDesc
                        TaxonomyFct activeFct = ((TaxonomyFct) amFct.getActiveFct(attrDesc));
                        Taxonomy taxonomy = activeFct.getTaxonomy();
                        // Get nodes in that taxonomy
                        List<TaxonomyNode> nodes = activeFct.getTopSymbol().getNodes();

                        // Loop through nodes to find current undesired ingredient
                        for (TaxonomyNode node : nodes) {
                            if (node.toString().equals(ingredient)) {

                                List<TaxonomyNode> allChildren = PreAdaptation.getAllChildren(node, taxonomy);
                                // add parent and its children to result list
                                for (TaxonomyNode child : allChildren) {
                                    resultList.add(child.toString());
                                }
                            }
                        }
                    }
                }
            }

        } catch (Exception e) {
//                    e.printStackTrace();
            System.out.println("Catched error in getChildrenOfIngredient");
        }

        return resultList;
    }

    // returns all children of all ingredients
    public static HashSet<String> getChildrenOfSeveralIngredients(HashSet<String> ingredients) {
        HashSet<String> result = new HashSet<String>();

        for (String ing : ingredients) {
            try {
                result.addAll(OperationsTaxonomy.getChildrenOfIngredient(ing));
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return result;
    }
}
