package no.ntnu.mycbr.rest.adaptation.helpClasses;

import de.dfki.mycbr.core.casebase.Attribute;
import de.dfki.mycbr.core.casebase.Instance;
import de.dfki.mycbr.core.model.AttributeDesc;
import de.dfki.mycbr.core.similarity.*;
import de.dfki.mycbr.util.Pair;
import no.ntnu.mycbr.CBREngine;
import no.ntnu.mycbr.rest.adaptation.adaptedInstance.AdaptationHistory;
import no.ntnu.mycbr.rest.adaptation.adaptedInstance.AdaptationProcess;
import no.ntnu.mycbr.rest.adaptation.rules.AdaptationSubstitution;

import java.util.HashMap;
import java.util.Map;

import java.util.*;

public class Print {

    public static void printUndesiredQueryProcess(String line) {
        if (Settings.printAddUnwantedIngredientChildren) {
            System.out.println(line);
        }
    }

    public static void printFiltering(String line) {
        if (Settings.printFilteredOriginalInstances) {
            System.out.println(line);
        }
    }

    public static void printRemoveEqualRecipesProcess(String line) {
        if (Settings.printRemoveEqualRecipesProcess) {
            System.out.println(line);
        }
    }

    public static void printPunishAdaptedRecipeProcess(String line) {
        if (Settings.printPunishAdaptedRecipes) {
            System.out.println(line);
        }
    }

    public static void printManualRetrieval(String line) {
        if (Settings.printManualRetrieval) {
            System.out.println(line);
        }
    }

    public static void printHistory(AdaptationHistory history) {
        for (String line: history.history) {
            System.out.println(line);
        }
    }

    public static void printSimpleAdaptationEvaluation(Pair<Instance, Similarity> originalResult, Pair<Instance, Similarity> adaptedResult,  Pair<Instance, Similarity> punishedAdaptedResult, AdaptationProcess adaptationProcess, Instance query) {
        if (Settings.printAdaptationEvaluation) {
            String original = originalResult.getSecond().getRoundedValue() + "";
            System.out.println(original.replace(".", ","));
            String adapted = adaptedResult.getSecond().getRoundedValue() + "";
            System.out.println(adapted.replace(".", ","));
        }
    }

    public static void printDetailedAdaptationEvaluation(Pair<Instance, Similarity> originalResult, Pair<Instance, Similarity> adaptedResult,  Pair<Instance, Similarity> punishedAdaptedResult, AdaptationProcess adaptationProcess, Instance query) {
        if (Settings.printAdaptationEvaluation) {
            System.out.println("");
            System.out.println(originalResult.getFirst().getName());
            String original = originalResult.getSecond().getRoundedValue() + "";
            System.out.println(original.replace(".", ","));
            String adapted = adaptedResult.getSecond().getRoundedValue() + "";
            System.out.println(adapted.replace(".", ","));
            //String punishedAdapted = punishedAdaptedResult.getSecond().getRoundedValue() + "";
            //System.out.println(punishedAdapted.replace(".", ","));
        }
    }

    public static void printNumberOfAttributeValues() {
        HashMap<String, AttributeDesc> allAttributeDescs = CBREngine.getConcept().getAllAttributeDescs();
        List<AmalgamationFct> availableAmalgamFcts = CBREngine.getConcept().getAvailableAmalgamFcts();
        for (Map.Entry<String, AttributeDesc> attributeDesc : allAttributeDescs.entrySet()) {
            AttributeDesc attrDesc = attributeDesc.getValue();
            if (attrDesc.getName().equals("Minor Ingredient") || !Settings.getTaxonomiesThatShouldNotBeConsidered().contains(attrDesc.getName())) {
                for (AmalgamationFct amFct : availableAmalgamFcts) {
                    if (amFct.getActiveFct(attrDesc).getClass().equals(TaxonomyFct.class)) {
                        TaxonomyFct activeFct = ((TaxonomyFct) amFct.getActiveFct(attrDesc));
                        //Taxonomy taxonomy = activeFct.getTaxonomy();
                        List<TaxonomyNode> nodes = activeFct.getTopSymbol().getNodes();
                        int size = nodes.size();
                    }
                }
            }
        }
    }

    public static void printOneProcess(Pair<Instance, Similarity> originalResult, Pair<Instance, Similarity> adaptedResult,  Pair<Instance, Similarity> punishedAdaptedResult, AdaptationProcess adaptationProcess, Instance query) {
        if (Settings.printOneProcess) {
            AdaptationHistory history = adaptationProcess.getHistory();
            HashMap<AttributeDesc, Attribute> attributes = history.originalInstance.getAttributes();
            AttributeDesc titleDesc = CBREngine.getConcept().getAttributeDesc("Title");
            String title = originalResult.getFirst().getAttForDesc(titleDesc).getValueAsString();

            System.out.println(" ");
            System.out.println(" ");
            System.out.println(" ");
            System.out.println("--------- Adaptation process for "+ originalResult.getFirst().getName() + " - " + title + "  ---------");
            System.out.println(" ");

            // print adapted instance history
            if (Settings.printHistory) {
                for (String line: history.history) {
                    System.out.println(line);
                }
            }
            System.out.println(" ");

            String format = "%-25s%-30s%-30s%-30s%-22s%-15s%-15s%n";
            System.out.printf(format, "Attribute", "Query", "Original", "Adapted", "Sim O-A", "Sim O-Q", "Sim A-Q");

            for (Map.Entry<AttributeDesc, Attribute> entry : attributes.entrySet()) {
                AttributeDesc attrDesc = entry.getKey();
                String category = attrDesc.getName();
                String queryValue = query.getAttForDesc(attrDesc).getValueAsString();
                String oldValue = entry.getValue().getValueAsString();
                String newValue = history.adaptedInstance.getAttForDesc(attrDesc).getValueAsString();
                Similarity simOrigAdapt = Similarity.get(0.0);
                Similarity simOrigQuery = Similarity.get(0.0);
                Similarity simAdaptQuery = Similarity.get(0.0);
                Similarity simAdaptQuery2 = Similarity.get(0.0);
                try {
                    simOrigAdapt = CBREngine.getProject().getLocalSimilarity(entry.getValue(), history.adaptedInstance.getAttForDesc(attrDesc), attrDesc);
                    simOrigQuery = CBREngine.getProject().getLocalSimilarity(query.getAttForDesc(attrDesc), entry.getValue(), attrDesc);
                    simAdaptQuery = CBREngine.getProject().getLocalSimilarity(query.getAttForDesc(attrDesc), history.adaptedInstance.getAttForDesc(attrDesc), attrDesc);
                    simAdaptQuery2 = CBREngine.getProject().getLocalSimilarity(history.adaptedInstance.getAttForDesc(attrDesc), query.getAttForDesc(attrDesc),  attrDesc);
                } catch(Exception e) {
                    System.out.println("Error in print one process");
                    e.printStackTrace();
                }
                System.out.printf(format, category, queryValue, oldValue, newValue, simOrigAdapt, simOrigQuery, simAdaptQuery + " (" + simAdaptQuery2 + ")");

            }
            System.out.println(" ");
            printSubstitutions(adaptationProcess);
            System.out.println(" ");
            System.out.println(originalResult.getFirst().getName() + ": Similarity score: " + originalResult.getSecond().getRoundedValue());
            System.out.println(adaptedResult.getFirst().getName() + ": Previous similarity score: " + adaptedResult.getSecond().getRoundedValue());
            System.out.println(punishedAdaptedResult.getFirst().getName() + ": Similarity score: " + punishedAdaptedResult.getSecond().getRoundedValue());
        }
    }

    public static void printInvalidProcess(List<AdaptationProcess> invalidAdaptationProcess) {
        if (Settings.printNonValidAdaptedInstances) {
            System.out.println("");
            System.out.println("");
            System.out.println("");
            System.out.println("");
            System.out.println("------------------------------------------");
            System.out.println("------ Invalid Adaptation histories ------");
            System.out.println("------------------------------------------");
            System.out.println("");
            printAllAdaptedInstances(invalidAdaptationProcess);
        }

    }

    public static void printZeroProcess(List<AdaptationProcess> zeroAdaptationProcess) {
        if (Settings.printZeroAdaptedInstances) {
            System.out.println("");
            System.out.println("");
            System.out.println("");
            System.out.println("");
            System.out.println("------------------------------------------");
            System.out.println("------ Zero Adaptation histories ------");
            System.out.println("------------------------------------------");
            System.out.println("");
            printAllAdaptedInstances(zeroAdaptationProcess);
        }

    }

    public static void printGlobalStatistics(AdaptationHistory globalStatistics) {
        if (Settings.printGlobalStatistics) {
            System.out.println(" ");
            System.out.println(" ");
            System.out.println(" ");
            System.out.println("---------------------------------------------");
            System.out.println("------------- Global Statistics -------------");
            System.out.println("---------------------------------------------");
            System.out.println(" ");
            Print.printStatistics(globalStatistics, true);
        }

    }

    public static void printFinalNumbers(int validAdaptationsSize, int zeroAdaptationsSize, int invalidAdaptationsSize) {
        if (Settings.printFinalNumbers) {
            System.out.println("");
            System.out.println("");
            System.out.println("");
            System.out.println("---------------------------------------------");
            System.out.println("--------------- Final numbers ---------------");
            System.out.println("---------------------------------------------");
            System.out.println("");
            System.out.println("Number of valid adaptation processes:       " + validAdaptationsSize);
            System.out.println("Number of invalid adaptation processes:     " + invalidAdaptationsSize);
            System.out.println("Number of 0-processes:                      " + zeroAdaptationsSize);
        }
    }

    public static void printAllAdaptedInstances(List<AdaptationProcess> adaptationProcessList) {
        System.out.println("");
        for (AdaptationProcess process : adaptationProcessList) {

            AttributeDesc titleDesc = CBREngine.getConcept().getAttributeDesc("Title");
            String title = process.getAdaptedInstance().getAttForDesc(titleDesc).getValueAsString();

            System.out.println(" ");
            System.out.println(" ");
            System.out.println(" ");
            System.out.println("--------- Adaptation process for "+ process.getAdaptedInstance().getName() + " - " + title + "  ---------");

            AdaptationHistory history = process.getHistory();
            // print adapted instance history
            if (Settings.printHistory) {
                for (String line: history.history) {
                    System.out.println(line);
                }
            }
            // print old - new: what changes are made
            System.out.println("");
            System.out.println("");
            Print.printPrettyResult(history);
            System.out.println("----------------------------------------------");

            // print substitutions made
            System.out.println("");
            System.out.println("");
            Print.printSubstitutions(process);
            // print statistics
            if (Settings.printStatistics) {
                System.out.println("");
                System.out.println("");
                Print.printStatistics(process.getHistory(), false);
            }

            System.out.println("");
            System.out.println("Is modified: " + process.getIsModified());
            System.out.println("Is valid:    " + process.getIsValid());
        }
    }
    private static void printPrettyResult(AdaptationHistory history) {
        HashMap<AttributeDesc, Attribute> attributes = history.originalInstance.getAttributes();
        System.out.println(" ");
        System.out.println(" ");
        System.out.println("--------- modifications overview ---------");
        System.out.println(" ");
        System.out.println(history.originalInstance.getName()+":");
        for (Map.Entry<AttributeDesc, Attribute> entry : attributes.entrySet()) {
            AttributeDesc attrDesc = entry.getKey();
            String category = attrDesc.getName();
            String oldValue = entry.getValue().getValueAsString();
            String newValue = history.adaptedInstance.getAttForDesc(attrDesc).getValueAsString();
            Similarity localSimilarity = Similarity.get(0.0);
            try {
                localSimilarity = CBREngine.getProject().getLocalSimilarity(entry.getValue(), history.adaptedInstance.getAttForDesc(attrDesc), attrDesc);
            } catch(Exception e) {
                System.out.println("Error in print pretty result");
                e.printStackTrace();
            }
            String format = "%-40s%-40s%-40s%s%n";
            System.out.printf(format, category, oldValue, newValue, localSimilarity);
        }
    }

    private static void printSubstitutions(AdaptationProcess process) {
        System.out.println("Substitutions:");
        for (AdaptationSubstitution s : process.getSubstitutions()) {
            System.out.println(s.getIngredient1() + " --> " + s.getIngredient2());
        }
    }

    public static void printStatistics(AdaptationHistory stats, Boolean isGlobal) {
        if (isGlobal && Settings.printGlobalStatistics) {
            Print.printStatisticsContent(stats);
        }
        else if (!isGlobal && Settings.printStatistics){
            System.out.println("Statistics:");
            Print.printStatisticsContent(stats);
        }
    }

    public static void printStatisticsContent(AdaptationHistory stats) {
        System.out.println("undesired - remove rule count:             " + stats.getUndesiredRemoveRuleCount());
        System.out.println("undesired - substitution rule count:       " + stats.getUndesiredSubstitutionRuleCount());
        System.out.println("undesired - similarity substitution count: " + stats.getUndesiredSimilaritySubstitutionCount());
        System.out.println(" ");
        System.out.println("desired - addition rule count:             " + stats.getDesiredAdditionRuleCount());
        System.out.println("desired - substitution rule count:         " + stats.getDesiredSubstitutionRuleCount());
        System.out.println("desired - similarity substitution count:   " + stats.getDesiredSimilaritySubstitutionCount());

    }
}
