package no.ntnu.mycbr.rest.adaptation.rules;

import no.ntnu.mycbr.rest.adaptation.helpClasses.Operations;
import no.ntnu.mycbr.rest.adaptation.helpClasses.OperationsTaxonomy;
import no.ntnu.mycbr.rest.adaptation.helpClasses.Settings;

import java.io.File;
import java.io.FileReader;
import java.io.BufferedReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;

public class RulesInit {
    private static Boolean printRules = Settings.printRules;

    public List<SubstitutionRule> substitutionRules;
    public List<SimpleRule> additionRules;
    public List<SimpleRule> deletionRules;
    public List<SimpleRule> suitableRules;
    public List<SimpleRule> titleRules;

    public RulesInit() {
        this.substitutionRules = getSubstitutionRules();
        this.additionRules = getAdditionRules();
        this.deletionRules = getDeletionRules();
        this.suitableRules = getSuitableRules();
        this.titleRules = getTitleRules();
    }

    public static List<SubstitutionRule> getSubstitutionRules() {
        RulesInit.pp("");
        RulesInit.pp("GETS SUBSTITUTION RULES");
        return RulesInit.readSubstitutionRules();
    }

    public static List<SimpleRule> getAdditionRules() {
        RulesInit.pp("");
        RulesInit.pp("GETS ADDITION RULES");
        return RulesInit.readSimpleRules(Settings.additionFileName);
    }

    public static List<SimpleRule> getSuitableRules() {
        RulesInit.pp("");
        RulesInit.pp("GETS SUITABLE INGREDIENTS RULES");
        return RulesInit.readSimpleRules(Settings.suitableIngredientsFilename);
    }

    public static List<SimpleRule> getDeletionRules() {
        RulesInit.pp("");
        RulesInit.pp("GETS DELETION RULES");
        return RulesInit.readSimpleRules(Settings.removalFileName);
    }

    public static List<SimpleRule> getTitleRules() {
        RulesInit.pp("");
        RulesInit.pp("GETS TITLE RULES");
        return RulesInit.readSimpleTitleRules(Settings.titleFileName);
    }

    private static List<SimpleRule> readSimpleTitleRules(String filename) {
        List<SimpleRule> rules = new ArrayList<SimpleRule>();
        try {
            File file = new File(filename);
            FileReader fileReader = new FileReader(file);
            BufferedReader bufferedReader = new BufferedReader(fileReader);
            String line;
            while ((line = bufferedReader.readLine()) != null) {
                if (!line.contains("//") && !line.contains("#") && line.length()>5 && line.contains("-")) {
                    try {
                        String[] requirementsAndAdditionalIngredient = line.split("->", 200);
                        String reqs = requirementsAndAdditionalIngredient[0];
                        String ingredient = requirementsAndAdditionalIngredient[1];
                        // Sets requirements if there are any
                        HashMap<String, HashSet<String>> requirements = new HashMap<String, HashSet<String>>();
                        if (reqs!=null && reqs.length()!=0) {
                            String[] reqsValues = requirementsAndAdditionalIngredient[0].split(",", 200);
                            requirements = RulesInit.getRequirements(reqsValues);
                        }

                        // Create SimpleRule
                        List<String> resultIngredients = new ArrayList<String>();
                        resultIngredients.add(ingredient);
                        SimpleRule simpleAddition = new SimpleRule(requirements, resultIngredients);
                        rules.add(simpleAddition);
                        RulesInit.pp("Simple Rule: " + requirements.size() + " requirements -> " + ingredient + " (" + resultIngredients.size() + ")");
                    } catch (Exception e){
                        System.out.println (e);
                        System.out.println ("Error reading from simplyAdding og rimplyRemove file: Fetching rule.");
                    }
                }
            }
            fileReader.close();

        } catch (Exception e) {
//            e.printStackTrace();
//            System.out.println("Error in rules - simple rule reading");
        }
        return rules;
    }

    private static List<SimpleRule> readSimpleRules(String filename) {
        List<SimpleRule> rules = new ArrayList<SimpleRule>();
        try {
            File file = new File(filename);
            FileReader fileReader = new FileReader(file);
            BufferedReader bufferedReader = new BufferedReader(fileReader);
            String line;
            while ((line = bufferedReader.readLine()) != null) {
                if (!line.contains("//") && !line.contains("#") && line.length()>5 && line.contains("-")) {
                    try {
                        String[] requirementsAndAdditionalIngredient = line.split("->", 200);
                        String reqs = requirementsAndAdditionalIngredient[0];
                        String ingredient = requirementsAndAdditionalIngredient[1];


                        // Sets requirements if there are any
                        HashMap<String, HashSet<String>> requirements = new HashMap<String, HashSet<String>>();


                        if (reqs!=null && reqs.length()!=0) {
                            String[] reqsValues = requirementsAndAdditionalIngredient[0].split(",", 200);
                            requirements = RulesInit.getRequirements(reqsValues);
                        }

                        // Create SimpleRule
                        List<String> resultIngredients = RulesInit.getResultForIngredient(ingredient);
                        if (resultIngredients != null && resultIngredients.size() != 0) {
                            SimpleRule simpleAddition = new SimpleRule(requirements, resultIngredients);
                            rules.add(simpleAddition);
                            RulesInit.pp("Simple Rule: " + requirements.size() + " requirements -> " + ingredient + " (" + resultIngredients.size() + ")");
                        }
                        else {
                            RulesInit.pp("Simple Rule: " + requirements.size() + " requirements -> " + ingredient + " (" + resultIngredients + ") is NULL or resultList is length 0");
                        }
                    } catch (Exception e){
//                        System.out.println (e);
//                        System.out.println ("Error reading from simplyAdding og rimplyRemove file: Fetching rule.");
                    }
                }
            }
            fileReader.close();

        } catch (Exception e) {
//            e.printStackTrace();
//            System.out.println("Error in rules - simple rule reading");
        }
        return rules;
    }

    public static List<SubstitutionRule> readSubstitutionRules() {
        List<SubstitutionRule> rules = new ArrayList<SubstitutionRule>();
        try {
            File file = new File(Settings.substitutionFileName);
            FileReader fileReader = new FileReader(file);
            BufferedReader bufferedReader = new BufferedReader(fileReader);
            String line;
            while ((line = bufferedReader.readLine()) != null) {
                if (line.length() < 2 || line.contains("//") || line.contains("#")) {
                    continue;
                }
                String[] requirementsAndSubstRules = line.split(";", 200);
                HashMap<String, HashSet<String>> requirements = new HashMap<String, HashSet<String>>();
                // Add requirements
                if (requirementsAndSubstRules[0] != null && requirementsAndSubstRules[0].length() > 2) {
                    // Ingredient requirements
                    String[] reqs = requirementsAndSubstRules[0].split(",", 200);
                    requirements = RulesInit.getRequirements(reqs);
                }
                // Add both directions rule(s)
                String substRules = requirementsAndSubstRules[1];
                if (substRules.length()>5 && substRules.contains("<->")) {
                    String[] values = substRules.split("<->", 200);
                    SubstitutionRule substitutionRule = RulesInit.getSubstitutionRule(values, requirements);
                    if (substitutionRule != null) {
                        substitutionRule.setBothDirections(true);
                        substitutionRule.setRequirements(requirements);
                        rules.add(substitutionRule);
                        //RulesInit.pp("SubstitutionRule Rule: " + substitutionRule.getIngredient1() + " <-> " + substitutionRule.getIngredient2() + ", reqs: " + requirements);
                    }
                }
                // Add single direction rule(s)
                else if (line.length()>5 && line.contains("->")) {
                    String[] values = substRules.split("->", 200);
                    SubstitutionRule substitutionRule = RulesInit.getSubstitutionRule(values, requirements);
                    if (substitutionRule != null) {
                        rules.add(substitutionRule);
                        substitutionRule.setBothDirections(false);
                        substitutionRule.setRequirements(requirements);
                        //RulesInit.pp("SubstitutionRule Rule: " + substitutionRule.getIngredient1() + " -> " + substitutionRule.getIngredient2() + ", reqs: " + requirements);
                    }
                }
            }
            fileReader.close();

        } catch (Exception e) {
//            e.printStackTrace();
            System.out.println("Error in rules - substitution rule reading");
        }
        return rules;
    }

    private static HashMap<String, HashSet<String>> getRequirements(String[] reqs) {
        HashMap<String, HashSet<String>> requirements = new HashMap<String, HashSet<String>>();

        for (String requiredIngredient : reqs) {
            HashSet<String> ingSet = new HashSet<String>();
            if (requiredIngredient.substring(requiredIngredient.length()-1).equals("*")) {
                ingSet.add(requiredIngredient);
            }
            else {
                try {
                    List<String> requiredIngredientPlusChildren = OperationsTaxonomy.getChildrenOfIngredient(requiredIngredient);
                    ingSet.addAll(requiredIngredientPlusChildren);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
            ingSet.add(requiredIngredient);
            requirements.put(requiredIngredient, ingSet);
        }
        return requirements;

    }

    private static SubstitutionRule getSubstitutionRule(String[] values, HashMap<String, HashSet<String>> reqs) {
        String ingredient1 = values[0].trim();
        String ingredient2 = values[1].trim();

        List<String> ingredientList1 = RulesInit.getResultForIngredient(ingredient1);
        List<String> ingredientList2 = RulesInit.getResultForIngredient(ingredient2);

        if (ingredientList1==null || ingredientList2 == null) {
            return null;
        }

        RulesInit.pp("Substitution rule: " + reqs.size() + " requirements; " + ingredient1 + " (" + ingredientList1.size() + ") -> " + ingredient2 + " (" + ingredientList2.size() + ")");
        return new SubstitutionRule(ingredientList1, ingredientList2);
    }

    // If ingredient ends with *: Return List containing only ingredient
    // If ingredient has no children: Return List containing only ingredient
    // Otherwise, return list with ingredient and all its children
    private static List<String> getResultForIngredient(String ingredient) {
        List<String> resultList = new ArrayList<>();

        try {
            List<String> ingredientPlusChildren = OperationsTaxonomy.getChildrenOfIngredient(ingredient);

            if (ingredient.substring(ingredient.length()-1).equals("*") && Operations.ingredientExistInModel(ingredient.substring(0, ingredient.length()-1))) {
                resultList.add(ingredient.substring(0,ingredient.length()-1));
            }
            else if (!Operations.ingredientExistInModel(ingredient)) {
                RulesInit.pp(ingredient + " does not exist in myCBR. So no substitution rules created");
                return null;
            }
            else if (ingredientPlusChildren.size()==0) {
                resultList.add(ingredient);
            }
            else {
                resultList = ingredientPlusChildren;
            }

        } catch(Exception e) {
//            e.printStackTrace();
        }
        return resultList;
    }

    private static void pp(String rule) {
        if (printRules) {
            System.out.println (rule);
        }
    }
}