package no.ntnu.mycbr.rest.adaptation.rules;

import no.ntnu.mycbr.rest.adaptation.adaptedInstance.AdaptationHistory;
import no.ntnu.mycbr.rest.adaptation.helpClasses.Settings;

import java.util.*;


// Simple Addition
// Simple Removal
public class SimpleRule extends Rule {
    public List<String> resultIngredients;

    public SimpleRule(HashMap<String, HashSet<String>> requirements, List<String> result) {
        this.requirements = requirements;
        this.resultIngredients = result;
    }

    // If one ingredient per MAIN requirement is in recipe, and
    // If resultIngredients contains desired ingredients, return true (ingredient can be simplyAdded / simply deleted)
    public Boolean requirementsAreSatisfied(HashSet<String> allIngredients, String ingredient) {
        return super.mainRequirementsAreSatisfied(allIngredients) && resultIngredients.contains(ingredient);
    }

    public String getTitleRule(HashSet<String> allIngredients) {
        if (super.mainRequirementsAreSatisfied(allIngredients) && resultIngredients.size()==1) {
            return resultIngredients.get(0);
        }
        return null;
    }

    // If one ingredient per MAIN requirement is in recipe, and
    // Requirements contains the new ingredient
    // Returns random amongst resultIngredients (suitable ingredient)
    public String suitableIngredientRequirementsAreSatisfied(HashSet<String> allIngredients, String newIngredient) {
        // check if main requirements are satisfied
        if (super.mainRequirementsAreSatisfied(allIngredients) && this.requirementsContainsIngredient(newIngredient)) {
            // the requirements must contain the new ingredient
            Random randomGenerator = new Random();
            int index = randomGenerator.nextInt(resultIngredients.size());
            return resultIngredients.get(index);
        }
        return null;
    }

    private Boolean requirementsContainsIngredient(String ingredient) {
        for (Map.Entry<String, HashSet<String>> entry : requirements.entrySet()) {
            HashSet<String> mainRequirementPlusChildren = entry.getValue();
            if (mainRequirementPlusChildren.contains(ingredient)) {
                return true;
            }
        }
        return false;
    }

    public void printRule(AdaptationHistory history, HashSet<String> allIngredients, String newIngredient) {
        if (Settings.printOneProcess) {
            System.out.println("");
            history.addLine("");
            history.addLine(".. main requirements are satisfied: " + super.mainRequirementsAreSatisfied(allIngredients));
            history.addLine(".. requirementscontains ingredient" + this.requirementsContainsIngredient(newIngredient));

            System.out.println("Requirements:");
            history.addLine("Requirements:");
            for (Map.Entry<String, HashSet<String>> entry : requirements.entrySet()) {
                System.out.println(entry.getKey() + "; " + entry.getValue().toString());
                history.addLine(entry.getKey() + "; " + entry.getValue().toString());
            }
            System.out.println("ResultValues");
            history.addLine("ResultValues");
            System.out.println(resultIngredients.toString());
            history.addLine(resultIngredients.toString());
        }
    }
}
