package no.ntnu.mycbr.rest.adaptation.rules;

import java.util.*;

public class SubstitutionRule extends Rule {
    private Boolean bothDirections = false;
    private List<String> ingredient1;
    private List<String> ingredient2;
    public void setBothDirections(Boolean bd) {this.bothDirections = bd;}

    public SubstitutionRule(List<String> ing1, List<String> ing2, HashMap<String, HashSet<String>>  requirements){
        this.ingredient1 = ing1;
        this.ingredient2 = ing2;
        this.requirements = requirements;
    }

    public SubstitutionRule(List<String> ing1, List<String> ing2){
        this.ingredient1 = ing1;
        this.ingredient2 = ing2;
        this.requirements = new HashMap<String, HashSet<String>>();
    }

    public void setRequirements(HashMap<String, HashSet<String>>  req) {
        requirements = req;
    }

    // Returns an ingredient we want to substitute IN when undesiredIngredient is substituted OUT
    public String getRemovalSubstitution(String undesiredIngredient, HashSet<String> allIngredients) {

        if (!super.mainRequirementsAreSatisfied(allIngredients)) {
            return null;
        }

        if (!bothDirections) {

            for (String ing1 : ingredient1) {
                for (String ing2: ingredient2) {
                    if (ing1.equals(undesiredIngredient) && !allIngredients.contains(ing2)) {
                        return ing2;
                    }
                }
            }
        }
        else {
            for (String ing1 : ingredient1) {
                for (String ing2: ingredient2) {
                    if (ing1.equals(undesiredIngredient) && !allIngredients.contains(ing2)) {
                        return ing2;
                    }
                    else if (ing2.equals(undesiredIngredient) && !allIngredients.contains(ing1)) {
                        return ing1;
                    }
                }
            }
        }
        return null;
    }

    // Returns an offer among the original ingredients so that we can get desired ingredient in
    public String getSubstitutionOffer(String desiredIngredient, HashSet<String> allIngredients, HashSet<String> allDesiredAndTheirChildren) {
        if (!this.mainRequirementsAreSatisfied(allIngredients)) {
            return null;
        }

        if (!bothDirections) {
            for (String ing1 : ingredient1) {
                for (String ing2: ingredient2) {
                    if ((ing2).equals(desiredIngredient)
                            && !allDesiredAndTheirChildren.contains(ing1)
                            && allIngredients.contains(ing1)) {
                        return ing1;
                    }
                }
            }

        }
        else {
            for (String ing1 : ingredient1) {
                for (String ing2 : ingredient2) {
                    // ingredient1 is query ingredient, ingredient2 is not in query, ingredient2 is in original
                    if (ing1.equals(desiredIngredient)
                            && !allDesiredAndTheirChildren.contains(ing2)
                            && allIngredients.contains(ing2)) {
                        return ing2;
                    }
                    // ingredient2 is query ingredient, ingredient1 is not in query, ingredient1 is in original
                    // salmon -> chicken
                    else if (ing2.equals(desiredIngredient)
                            && !allDesiredAndTheirChildren.contains(ing1)
                            && allIngredients.contains(ing1)) {
                        return ing1;
                    }
                }
            }

        }
        return null;
    }

    public String printRule() {
        return ("Ing1: " + this.ingredient1 + ", Ing2: " + this.ingredient2);
    }
}