import {Component, OnInit} from "@angular/core";
import {HomeService} from "./home.service";
import {SearchComponent} from "./search.component";
import {Case, KeyValuePair, Recipe} from "./home.model";
import {GlobalFunctions} from "./shared/globalfunctions";
import {AppSettings} from "../../appSettings";
import "rxjs/add/operator/mergeMap";

@Component({
  selector: 'home',
  templateUrl: 'app/components/home/home.component.html'
})

export class HomeComponent implements OnInit {

  globalFunctions: GlobalFunctions = new GlobalFunctions;

  attributes: KeyValuePair[] = [];
  values: KeyValuePair[] = [];

  desiredSearch: SearchComponent = new SearchComponent;
  undesiredSearch: SearchComponent = new SearchComponent;

  searchedDesired: KeyValuePair[] = [];
  searchedUndesired: KeyValuePair[] = [];

  similarCases: Case[] = [];
  noCasesFoundMessage: string;

  modalCase: Case;
  modalReturnMessage: string;

  casesAddedToCasebase: string[] = [];

  // For development settings panel
  doAdaptation: boolean;
  showAdaptation: boolean;
  showCaseDetails: boolean;
  showPreparation: boolean;
  numberOfCases: number;
  casebaseNames: string[];
  casebaseName: string;

  static sortRecipesByScore(caseList: Case[]) {
    caseList.sort(function (a, b) {
      return (a.score < b.score) ? 1 : ((b.score < a.score) ? -1 : 0);
    });
    return caseList;
  }

  static arraysAreEqual(a1: string[], a2: string[]) {
    return JSON.stringify(a1) === JSON.stringify(a2);
  }

  static getIngredientRemoval(ingredient: string, substitutions: KeyValuePair[]): boolean {
    for (let substitution of substitutions) {
      if (((substitution.value === '') || (substitution.value.toLowerCase() === 'x'))
        && substitution.key.toLowerCase() === ingredient.toLowerCase()) {
        return true;
      }
    }
    return false;
  }

  static generateNewCaseAttributesString(adaptedCase: Case) {
    let nameWithoutAdapted = adaptedCase.name.replace('-Adapted', '');
    adaptedCase.attributeDict.push(new KeyValuePair('CaseName', nameWithoutAdapted));

    let adaptedCaseQuery = '{';
    for (let i = 0; i < adaptedCase.attributeDict.length; i++) {
      if (i === adaptedCase.attributeDict.length - 1) {
        adaptedCaseQuery += '"' + adaptedCase.attributeDict[i].key + '": "' + adaptedCase.attributeDict[i].value + '"}';
      } else {
        adaptedCaseQuery += '"' + adaptedCase.attributeDict[i].key + '": "' + adaptedCase.attributeDict[i].value + '",';
      }
    }
    return adaptedCaseQuery;
  }

  // Checks if two lists of KeyValuePairs contains the same ingredients
  findDifferenceInKeyValuePair(checkIngredients: KeyValuePair[],
                               recipeIngredients: KeyValuePair[],
                               shouldBePrinted: boolean,
                               shouldNotBeChecked: boolean): string[] {

    if (shouldNotBeChecked === false) { // Adapted cases should not be checked because the correct recipe does not exist yet
      let simpleIngredients: string[] = [];

      for (let u = 0; u < recipeIngredients.length; u++) {
        simpleIngredients.push(recipeIngredients[u].key.toLowerCase());
      }

      let simpleValues: string[] = [];
      for (let k = 0; k < checkIngredients.length; k++) {
        if (checkIngredients[k].key === 'Basic'
          || checkIngredients[k].key === 'Fruit'
          || checkIngredients[k].key === 'Minor Ingredient'
          || checkIngredients[k].key === 'Meat'
          || checkIngredients[k].key === 'Spice and Herb'
          || checkIngredients[k].key === 'Supplement'
          || checkIngredients[k].key === 'Fish'
          || checkIngredients[k].key === 'Carbs'
          || checkIngredients[k].key === 'Drinks'
          || checkIngredients[k].key === 'Oil and Fat'
          || checkIngredients[k].key === 'Vegetable'
          || checkIngredients[k].key === 'Milk') {
          if (checkIngredients[k].value !== '_unknown_') {
            let currentValues = checkIngredients[k].value.split(';');
            currentValues.pop();
            for (let y = 0; y < currentValues.length; y++) {
              simpleValues.push(currentValues[y].toLowerCase());
            }
          }
        }
      }

      let differentIngredients: string[] = [];

      if (!HomeComponent.arraysAreEqual(simpleIngredients.sort(), simpleValues.sort())) {

        if (shouldBePrinted === true) {
          console.log('\n%cThere is a case with different ingredients in recipe versus case:', 'color: red');

          // console.log('All ingredients in recipe: ' + simpleIngredients.sort());
          // console.log('All ingredients in casebase: ' + simpleValues.sort());

          console.log('Ingredients only in recipe: ' + simpleIngredients.filter(function (x) {
              return simpleValues.indexOf(x) < 0;
            }));

          console.log('Ingredients only in casebase: ' + simpleValues.filter(function (x) {
              return simpleIngredients.indexOf(x) < 0;
            }) + '\n\n');
        }

        differentIngredients = simpleIngredients.filter(function (x) {
          return simpleValues.indexOf(x) < 0;
        });
      }

      return differentIngredients;
    }
  }

  constructor(private homeService: HomeService) {
  }

  ngOnInit() {
    this.showAdaptation = AppSettings.SHOW_ADAPTATIONS;
    this.doAdaptation = AppSettings.DO_ADAPTATION;
    this.showCaseDetails = AppSettings.SHOW_DEVELOPER_HELP;
    this.showPreparation = AppSettings.SHOW_CASE_DETAILS;
    this.numberOfCases = AppSettings.NUMBER_OF_CASES;
    this.casebaseNames = AppSettings.CASEBASE_NAMES;
    this.casebaseName = AppSettings.CASEBASE_NAMES[0];
    this.retrieveAttributesAndValues();
  }

  retrieveAttributesAndValues() {
    this.homeService.getAttributes()

      .subscribe(retrievedAttributes => {
          this.attributes = retrievedAttributes;

          for (let i = 0; i < retrievedAttributes.length; i++) {
            this.homeService.getValuesForAttribute(retrievedAttributes[i].key)

              .subscribe(retrievedValues => {

                  if (retrievedValues !== undefined) {
                    for (let j = 0; j < retrievedValues.length; j++) {
                      this.values = this.values.concat(new KeyValuePair(retrievedValues[j], retrievedAttributes[i].key));
                    }
                  }
                }
              );
          }
        }
      );
  }

  retrieveSimilarCases() {
    let resultCases: Case[] = [];
    let tempCases: Case[];

    this.homeService.getSimilarCases(this.casebaseName, this.desiredSearch.generateQueryString(),
      this.undesiredSearch.generateQueryString(),
      this.doAdaptation)
      .subscribe(
        retrievedCases => {

          if (retrievedCases.length === 0) {
            this.noCasesFoundMessage = 'The system could not find any recipes to match your desires.';
            for (let queryUndesiredValue of this.undesiredSearch.selected) {
              if (queryUndesiredValue.key.toLowerCase() === 'bread') {
                this.noCasesFoundMessage += ' The database of recipes only contains sandwich recipes, which all contains bread.';
              }
            }
            this.similarCases = [];
          } else {
            this.noCasesFoundMessage = '';
            tempCases = retrievedCases;

            for (let tempCase of tempCases) {

              // TODO when developing: skips cases with name containing 'query' - - - - - - - - - - - - - - - - - - - - -
              if (tempCase.name.lastIndexOf('query', 0) === 0) {
                console.log('Skipping case with name ' + tempCase.name);
                continue;
              }
              // END - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

              if (tempCase.score === 0) {
                continue;
              }

              this.homeService.getCase(this.casebaseName, tempCase.name).flatMap(retrievedDetails => {
                tempCase.setAttributes(retrievedDetails);

                let caseId = tempCase.id;
                if ((tempCase.name.indexOf('-') !== -1
                  && (!isNaN(parseInt(tempCase.name.substr(tempCase.name.length - 1), 10))))
                  || ((tempCase.name.split('-').length - 1) > 1)) {
                  caseId += '-' + tempCase.name.split(/-(.+)/)[1].replace('-Adapted', '');
                }

                return this.homeService.getRecipe(caseId);
              }).subscribe(retrievedRecipe => {
                  tempCase.recipe = retrievedRecipe;

                  this.findDifferenceInKeyValuePair(tempCase.attributeDict, tempCase.recipe.ingredients, true, tempCase.adapted);

                  tempCase.addMissingIngredients(this.findDifferenceInKeyValuePair(tempCase.attributeDict,
                    this.desiredSearch.selected, false, false));
                  resultCases.push(tempCase);
                },
                error => console.log(error),
                () => {
                  this.similarCases = HomeComponent.sortRecipesByScore(resultCases).slice(0, this.numberOfCases);
                  this.casesAddedToCasebase = [];
                  this.searchedDesired = this.desiredSearch.selected.slice();
                  this.searchedUndesired = this.undesiredSearch.selected.slice();
                });
            }
          }
        }
      );
  }

  isCaseAlreadyAddedToCasebase(adaptedCaseName: string) {
    return this.casesAddedToCasebase.indexOf(adaptedCaseName) !== -1;
  }

  addNewCaseToCasebaseAndRecipebase(adaptedCase: Case) {
    const adaptedRecipeClone: Recipe = JSON.parse(JSON.stringify(adaptedCase.recipe)); // Hack to deep copy old object
    adaptedRecipeClone.ingredients = this.adaptRecipeIngredients(adaptedCase.recipe, adaptedCase.adaptations);
    adaptedRecipeClone.createdByAI = true;
    let adaptedRecipeString: string = JSON.stringify(adaptedRecipeClone).replace(/"key"/g, '"model"').replace(/"value"/g, '"display"');
    this.homeService.addCase(this.casebaseName,
      HomeComponent.generateNewCaseAttributesString(adaptedCase),
      adaptedRecipeString).subscribe(response => {
      if (response._body === 'casebase') {
        this.modalReturnMessage = 'Failed to add the new case to our database. Please try again.';
      } else if (response._body === 'recipe') {
        this.modalReturnMessage = 'Failed to add the new recipe to our database. Please try again.';
      } else if (response._body === 'success') {
        this.modalReturnMessage = 'The recipe was successfully added to our database.';
        this.casesAddedToCasebase.push(adaptedCase.name);
      }
    });
  }

  adaptRecipeIngredients(oldRecipe: Recipe, adaptations: KeyValuePair[]): KeyValuePair[] {
    let adaptedIngredients: KeyValuePair[] = [];

    for (let ingredient of this.getIngredientAdditions(adaptations)) {
      adaptedIngredients.push(new KeyValuePair(ingredient, ingredient));
    }
    for (let ingredient of oldRecipe.ingredients) {
      if (!HomeComponent.getIngredientRemoval(ingredient.key, adaptations)) {
        let ingredientSubstitute: string = (this.getIngredientSubstitution(ingredient.key, adaptations));
        if (ingredientSubstitute !== '') {
          adaptedIngredients.push(new KeyValuePair(ingredientSubstitute, ingredientSubstitute));
        } else {
          adaptedIngredients.push(ingredient);
        }
      }
    }
    return adaptedIngredients;
  }

  getIngredientAdditions(substitutions: KeyValuePair[]): string[] {
    let ingredientAdditions: string[] = [];
    for (let substitution of substitutions) {
      if (substitution.key.length === 0 || substitution.key.toLowerCase() === 'x') {
        ingredientAdditions.push(substitution.value);
      }
    }
    return ingredientAdditions;
  }

  getIngredientSubstitution(ingredient: string, substitutions: KeyValuePair[]): string {
    for (let substitution of substitutions) {
      if (substitution.key.toLowerCase() === ingredient.toLowerCase()) {
        return substitution.value;
      }
    }
    return '';
  }

  getIngredientParents(ingredient: string, queryParentRecipeChild: KeyValuePair[]): string[] {
    let ingredientParents: string[] = [];
    for (let child of queryParentRecipeChild) {
      if (ingredient.toLowerCase() === child.value.toLowerCase()) {
        ingredientParents.push(child.key);
      }
    }
    return ingredientParents;
  }

  addPreparationLinksToRecipe(step: string) {
    let preparationLinks = HomeService.getPreparationLinks();
    let resultStep = step;
    for (let k = 0; k < preparationLinks.length; k++) {
      let index = resultStep.toLowerCase().indexOf(preparationLinks[k].key.toString());
      if (index !== -1) {
        resultStep = resultStep.substr(0, index)
          + '<a href=\"' + preparationLinks[k].value
          + '\" target=\"_blank\">'
          + resultStep.substr(index, preparationLinks[k].key.length)
          + '</a>'
          + resultStep.substr(index + preparationLinks[k].key.length);
      }
    }
    return resultStep;
  }

  setModalCase(modalCase: Case) {
    this.modalCase = modalCase;
  }
}
