/**
 * Created by martheoynes on 13.02.2017.
 */

import {KeyValuePair} from "./home.model";
import {GlobalFunctions} from "./shared/globalfunctions";

export class SearchComponent {

  filtered: KeyValuePair[] = [];
  selected: KeyValuePair[] = [];
  hoverSelected: KeyValuePair;

  animateSelectedValues: Boolean = false;

  globalFunctions: GlobalFunctions = new GlobalFunctions();

  filter(rawQuery: string, searchValues: KeyValuePair[], valuesToExclude: KeyValuePair[]) {
    if (rawQuery !== '') {
      this.filtered = searchValues.filter(function (value: KeyValuePair) {
        return (value.key.toLowerCase().substr(0, rawQuery.length) === rawQuery.toLowerCase()) === true
          && this.globalFunctions.findIndexOfElement(this.selected, value.key) === -1
          && this.globalFunctions.findIndexOfElement(valuesToExclude, value.key) === -1;

        // To search for general substring terms:
        // value.key.toLowerCase().indexOf(rawQuery.toLowerCase()) > -1;

        // To search for a substring that starts with:
        // value.key.toLowerCase().substr(0, rawQuery.length) === rawQuery.toLowerCase()) === true

      }.bind(this)).sort(function (a, b) {
        return a.key.localeCompare(b.key);
      });
    } else {
      this.filtered = [];
    }
  }

  select(item: KeyValuePair) {
    this.animateSelectedValues = true;
    this.selected.push(item);
    this.filtered = [];
  }

  remove(item: KeyValuePair) {
    for (let i = 0; i < this.selected.length; i++) {
      if (this.selected[i].key === item.key) {
        this.selected.splice(this.selected.indexOf(this.selected[i]), 1);
        break;
      }
    }
  }

  generateQueryString() {
    this.selected.sort(function (a, b) {
      return (a.value < b.value) ? 1 : ((b.value < a.value) ? -1 : 0);
    });

    let alreadyAddedAttributes: string[] = [];
    let query = '';

    if (this.selected.length > 0) {
      query += '{';
      for (let i = 0; i <= this.selected.length - 1; i++) {
        if (alreadyAddedAttributes.length < 1) {
          query += '"' + this.selected[i].value + '": "' + this.selected[i].key;
          alreadyAddedAttributes.push(this.selected[i].value);
        } else {
          if (alreadyAddedAttributes.indexOf(this.selected[i].value) !== -1) {
            query += ',' + this.selected[i].key;
          } else {
            query += '", "' + this.selected[i].value + '": "' + this.selected[i].key;
            alreadyAddedAttributes.push(this.selected[i].value);
          }
        }
        if (i === this.selected.length - 1) {
          query += '"}';
        }
      }
    } else {
      return '{}';
    }
    return query;
  }

  handleKey(keyEvent: any) {
    if (keyEvent.keyCode === 8) { // Backspace
      if (this.filtered.length === 0) {
        this.hoverSelected = null;
      } else {
        this.hoverSelected = this.filtered[0];
      }
    } else if (keyEvent.keyCode === 40) { // Down
      if (this.hoverSelected == null) {
        this.hoverSelected = this.filtered[0];
      } else {
        let currentIndex = this.globalFunctions.findIndexOfElement(this.filtered, this.hoverSelected.key);
        if (currentIndex !== this.filtered.length - 1) {
          this.hoverSelected = this.filtered[currentIndex + 1];
        }

      }
    } else if (keyEvent.keyCode === 38) { // Up
      if (this.hoverSelected != null) {
        let currentIndex = this.globalFunctions.findIndexOfElement(this.filtered, this.hoverSelected.key);
        if (currentIndex !== 0) {
          this.hoverSelected = this.filtered[currentIndex - 1];
        }
      }
    } else if (keyEvent.keyCode === 13) { // Enter
      this.animateSelectedValues = true;
      if (this.hoverSelected == null) {
        if (this.filtered.length > 0) {
          this.selected.push(this.filtered[0]);
        }
      } else {
        this.selected.push(this.hoverSelected);
        this.hoverSelected = null;
      }
    } else { // Everything else
      this.hoverSelected = this.filtered[0];
    }
  }
}
