import {Component, OnInit} from "@angular/core";
import {Case} from "../home/home.model";
import {HomeService} from "../home/home.service";
import {GlobalFunctions} from "../home/shared/globalfunctions";
import {QuizService} from "./quiz.service";
import {AppSettings} from "../../appSettings";

@Component({
  selector: 'botornot',
  templateUrl: 'app/components/quiz/quiz.component.html'
})

export class QuizComponent implements OnInit {
  globalFunctions: GlobalFunctions = new GlobalFunctions;
  quizCases: Case[] = [];
  currentCase: Case;
  responseToUser: string = '';
  displayStartbutton = true;
  displayAnswerButtons = false;
  displayNextButtons = false;
  quizHasStarted = false;

  constructor(private homeService: HomeService,
              private quizService: QuizService) {
  }

  ngOnInit() {
    this.retrieveAllCases();
  }

  shuffleList() {
    let i: number;
    let j = 0;
    let temp: any = null;

    for (i = this.quizCases.length - 1; i > 0; i -= 1) {
      j = Math.floor(Math.random() * (i + 1));
      temp = this.quizCases[i];
      this.quizCases[i] = this.quizCases[j];
      this.quizCases[j] = temp;
    }
  }

  startQuiz() {
    this.shuffleList();
    this.quizHasStarted = true;
    this.currentCase = this.quizCases.pop();
    this.displayAnswerButtons = true;
    this.displayStartbutton = false;
  }

  nextTask() {
    if (this.quizCases.length > 0) {
      this.responseToUser = '';
      this.displayAnswerButtons = true;
      this.displayNextButtons = false;
      this.currentCase = this.quizCases.pop();
    } else {
      this.responseToUser = 'There are no more test cases. Thank you for your help!';
      this.displayAnswerButtons = false;
      this.displayNextButtons = false;
      this.currentCase = this.quizCases.pop();
    }
  }

  submitAnswer(guess: boolean) {
    if (this.currentCase.recipe.createdByAI === guess) {
      if (this.currentCase.recipe.createdByAI === true) {
        this.responseToUser = 'Correct! The recipe is created by a computer.';
      } else {
        this.responseToUser = 'Correct! The recipe is created by a human.';
      }
      this.quizService.submitAnswer(this.currentCase.name, this.currentCase.recipe.createdByAI, 'correct').subscribe();
    } else {
      if (this.currentCase.recipe.createdByAI === true) {
        this.responseToUser = 'Wrong. The recipe is created by a computer.';
      } else {
        this.responseToUser = 'Wrong. The recipe is created by a human.';
      }
      this.quizService.submitAnswer(this.currentCase.name, this.currentCase.recipe.createdByAI, 'wrong').subscribe();
    }
    this.displayNextButtons = true;
    this.displayAnswerButtons = false;
  }

  retrieveAllCases() {
    let tempCases: Case[];

    this.homeService.getSimilarCases(AppSettings.QUIZ_CASEBASE, '{"Carbs":"bread"}', '{}', false) // Hack to retrieve all recipes
      .subscribe(
        retrievedCases => {

          if (retrievedCases.length === 0) {
            this.quizCases = [];
          } else {
            tempCases = retrievedCases;

            for (let tempCase of tempCases) {

              // TODO when developing: skips cases with name containing 'query' - - - - - - - - - - - - - - - - - - - - -
              if (tempCase.name.lastIndexOf('query', 0) === 0) {
                console.log('Skipping case with name ' + tempCase.name);
                continue;
              }
              // END - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

              this.homeService.getCase(AppSettings.QUIZ_CASEBASE, tempCase.name).flatMap(retrievedDetails => {
                tempCase.setAttributes(retrievedDetails);

                let caseId = tempCase.id;
                if ((tempCase.name.indexOf('-') !== -1
                  && (!isNaN(parseInt(tempCase.name.substr(tempCase.name.length - 1), 10))))
                  || ((tempCase.name.split('-').length - 1) > 1)) {
                  caseId += '-' + tempCase.name.split(/-(.+)/)[1].replace('-Adapted', '');
                }

                return this.homeService.getRecipe(caseId);
              }).subscribe(retrievedRecipe => {
                tempCase.recipe = retrievedRecipe;
                this.quizCases.push(tempCase);
              });
            }
          }
        }
      );
  }

}
